@@ -52,7 +52,7 @@ struct contextMeshOptions {
   int partitionCreatePhysicals, partitionSplitMeshFiles,
     partitionSaveTopologyFile;
   int partitionTriWeight, partitionQuaWeight, partitionTetWeight,
-    partitionHexWeight;
+    partitionHexWeight, partitionLinWeight;
   int partitionPriWeight, partitionPyrWeight, partitionTrihWeight;
   int partitionOldStyleMsh2;
   int metisAlgorithm, metisEdgeMatching, metisRefinementAlgorithm;
