@@ -1,55 +1,60 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _GMSH_DEFINES_H_
 #define _GMSH_DEFINES_H_
 
 // IO file formats (numbers should not be changed)
-#define FORMAT_MSH   1
-#define FORMAT_UNV   2
-#define FORMAT_GREF  3
-#define FORMAT_XPM   4
-#define FORMAT_PS    5
-#define FORMAT_BMP   6
-#define FORMAT_GIF   7
-#define FORMAT_GEO   8
-#define FORMAT_JPEG  9
-#define FORMAT_AUTO  10
-#define FORMAT_PPM   11
-#define FORMAT_YUV   12
-#define FORMAT_DMG   13
-#define FORMAT_SMS   14
-#define FORMAT_OPT   15
-#define FORMAT_VTK   16
-#define FORMAT_MPEG  17
-#define FORMAT_TEX   18
-#define FORMAT_VRML  19
-#define FORMAT_EPS   20
-#define FORMAT_MAIL  21
-#define FORMAT_PNG   22
-#define FORMAT_TXT   23
-#define FORMAT_PDF   24
-#define FORMAT_RMED  25
-#define FORMAT_POS   26
-#define FORMAT_STL   27
-#define FORMAT_P3D   28
-#define FORMAT_SVG   29
-#define FORMAT_MESH  30
-#define FORMAT_BDF   31
-#define FORMAT_CGNS  32
-#define FORMAT_MED   33
-#define FORMAT_DIFF  34
-#define FORMAT_BREP  35
-#define FORMAT_STEP  36
-#define FORMAT_IGES  37
-#define FORMAT_IR3   38
-#define FORMAT_INP   39
-#define FORMAT_PLY2  40
-#define FORMAT_CELUM 41
-#define FORMAT_SU2   42
+#define FORMAT_MSH          1
+#define FORMAT_UNV          2
+#define FORMAT_XPM          4
+#define FORMAT_PS           5
+#define FORMAT_BMP          6
+#define FORMAT_GIF          7
+#define FORMAT_GEO          8
+#define FORMAT_JPEG         9
+#define FORMAT_AUTO         10
+#define FORMAT_PPM          11
+#define FORMAT_YUV          12
+#define FORMAT_OPT          15
+#define FORMAT_VTK          16
+#define FORMAT_MPEG         17
+#define FORMAT_TEX          18
+#define FORMAT_VRML         19
+#define FORMAT_EPS          20
+#define FORMAT_MAIL         21
+#define FORMAT_PNG          22
+#define FORMAT_TXT          23
+#define FORMAT_PDF          24
+#define FORMAT_RMED         25
+#define FORMAT_POS          26
+#define FORMAT_STL          27
+#define FORMAT_P3D          28
+#define FORMAT_SVG          29
+#define FORMAT_MESH         30
+#define FORMAT_BDF          31
+#define FORMAT_CGNS         32
+#define FORMAT_MED          33
+#define FORMAT_DIFF         34
+#define FORMAT_BREP         35
+#define FORMAT_STEP         36
+#define FORMAT_IGES         37
+#define FORMAT_IR3          38
+#define FORMAT_INP          39
+#define FORMAT_PLY2         40
+#define FORMAT_CELUM        41
+#define FORMAT_SU2          42
 #define FORMAT_MPEG_PREVIEW 43
+#define FORMAT_PGF          44
+#define FORMAT_PVTU         45
+#define FORMAT_X3D          46
+#define FORMAT_TOCHNOG      47
+#define FORMAT_TIKZ         48
+#define FORMAT_NEU          49
+#define FORMAT_MATLAB       50
+#define FORMAT_KEY          51
 
 // Element types
 #define TYPE_PNT     1
@@ -63,6 +68,8 @@
 #define TYPE_POLYG   9
 #define TYPE_POLYH   10
 #define TYPE_XFEM    11
+#define TYPE_MINI    12
+#define TYPE_TRIH    13
 
 // Element types in .msh file format (numbers should not be changed)
 #define MSH_LIN_2    1
@@ -209,26 +216,29 @@
 #define MSH_TRI_SUB 135
 #define MSH_TET_SUB 136
 #define MSH_TET_16  137
+#define MSH_TRI_MINI 138
+#define MSH_TET_MINI 139
+#define MSH_TRIH_4   140
 
-#define MSH_NUM_TYPE 137
+#define MSH_NUM_TYPE 140
 
 // Geometric entities
 #define ENT_NONE     0
 #define ENT_POINT    (1<<0)
-#define ENT_LINE     (1<<1)
+#define ENT_CURVE    (1<<1)
 #define ENT_SURFACE  (1<<2)
 #define ENT_VOLUME   (1<<3)
-#define ENT_ALL      (ENT_POINT | ENT_LINE | ENT_SURFACE | ENT_VOLUME)
+#define ENT_ALL      (ENT_POINT | ENT_CURVE | ENT_SURFACE | ENT_VOLUME)
 
 // 2D meshing algorithms (numbers should not be changed)
 #define ALGO_2D_MESHADAPT      1
 #define ALGO_2D_AUTO           2
-#define ALGO_2D_MESHADAPT_OLD  4
 #define ALGO_2D_DELAUNAY       5
 #define ALGO_2D_FRONTAL        6
 #define ALGO_2D_BAMG           7
 #define ALGO_2D_FRONTAL_QUAD   8
 #define ALGO_2D_PACK_PRLGRMS   9
+#define ALGO_2D_PACK_PRLGRMS_CSTR   10
 
 // 3D meshing algorithms (numbers should not be changed)
 #define ALGO_3D_DELAUNAY       1
@@ -237,6 +247,7 @@
 #define ALGO_3D_FRONTAL_HEX    6
 #define ALGO_3D_MMG3D          7
 #define ALGO_3D_RTREE          9
+#define ALGO_3D_HXT           10
 
 // Meshing methods
 #define MESH_NONE         0
