@@ -996,7 +996,7 @@ int meshStatFileDialog(const char *name)
     dialog->window->box(GMSH_WINDOW_BOX);
     dialog->window->set_modal();
     dialog->b[0] = new Fl_Check_Button(WB, y, 2 * BBB + WB, BH,
-                                       "Save all (ignore physical groups)");
+                                       "Save all elements");
     y += BH;
     dialog->b[1] =
       new Fl_Check_Button(WB, y, 2 * BBB + WB, BH, "Print elementary tags");
@@ -1086,10 +1086,8 @@ int mshFileDialog(const char *name)
     {"Version 1", 0, 0, 0},
     {"Version 2 ASCII", 0, 0, 0},
     {"Version 2 Binary", 0, 0, 0},
-    {"Version 3 ASCII (Experimental)", 0, 0, 0},
-    {"Version 3 Binary (Experimental)", 0, 0, 0},
-    {"Version 4 ASCII (Experimental)", 0, 0, 0},
-    {"Version 4 Binary (Experimental)", 0, 0, 0},
+    {"Version 4 ASCII", 0, 0, 0},
+    {"Version 4 Binary", 0, 0, 0},
     {0}};
 
   int BBB = BB + 9; // labels too long
@@ -1106,7 +1104,7 @@ int mshFileDialog(const char *name)
     dialog->c->align(FL_ALIGN_RIGHT);
     dialog->c->callback((Fl_Callback *)format_cb, dialog);
     dialog->b[0] = new Fl_Check_Button(WB, y, 2 * BBB + WB, BH,
-                                       "Save all (ignore physical groups)");
+                                       "Save all elements");
     y += BH;
     dialog->b[0]->type(FL_TOGGLE_BUTTON);
     dialog->b[1] = new Fl_Check_Button(WB, y, 2 * BBB + WB, BH,
@@ -1130,12 +1128,10 @@ int mshFileDialog(const char *name)
 
   if(CTX::instance()->mesh.mshFileVersion == 1.0)
     dialog->c->value(0);
-  else if(CTX::instance()->mesh.mshFileVersion < 3.0)
+  else if(CTX::instance()->mesh.mshFileVersion < 4.0)
     dialog->c->value(!CTX::instance()->mesh.binary ? 1 : 2);
-  else if(CTX::instance()->mesh.mshFileVersion == 3.0)
-    dialog->c->value(!CTX::instance()->mesh.binary ? 3 : 4);
   else
-    dialog->c->value(!CTX::instance()->mesh.binary ? 5 : 6);
+    dialog->c->value(!CTX::instance()->mesh.binary ? 3 : 4);
   dialog->b[0]->value(CTX::instance()->mesh.saveAll ? 1 : 0);
   dialog->b[1]->value(CTX::instance()->mesh.saveParametric ? 1 : 0);
   dialog->b[2]->value(CTX::instance()->mesh.partitionSplitMeshFiles ? 1 : 0);
@@ -1157,11 +1153,9 @@ int mshFileDialog(const char *name)
           (dialog->c->value() == 0) ?
             1.0 :
             (dialog->c->value() == 1 || dialog->c->value() == 2) ?
-            2.2 :
-            (dialog->c->value() == 3 || dialog->c->value() == 4) ? 3.0 : 4.0);
+            2.2 : 4.0);
         opt_mesh_binary(0, GMSH_SET | GMSH_GUI,
-                        (dialog->c->value() == 2 || dialog->c->value() == 4 ||
-                         dialog->c->value() == 6) ?
+                        (dialog->c->value() == 2 || dialog->c->value() == 4) ?
                           1 :
                           0);
         opt_mesh_save_all(0, GMSH_SET | GMSH_GUI,
@@ -1188,8 +1182,7 @@ int mshFileDialog(const char *name)
 void format_cb(Fl_Widget *widget, void *data)
 {
   _mshFileDialog *dialog = static_cast<_mshFileDialog *>(data);
-  if((dialog->c->value() == 5 || dialog->c->value() == 6 ||
-      dialog->c->value() == 3 || dialog->c->value() == 4 ||
+  if((dialog->c->value() == 3 || dialog->c->value() == 4 ||
       dialog->c->value() == 1 || dialog->c->value() == 2) &&
      GModel::current()->getNumPartitions() > 0) {
     dialog->b[2]->activate();
@@ -1221,7 +1214,7 @@ int unvinpFileDialog(const char *name, const char *title, int format)
     dialog->window->box(GMSH_WINDOW_BOX);
     dialog->window->set_modal();
     dialog->b[0] = new Fl_Check_Button(WB, y, 2 * BBB + WB, BH,
-                                       "Save all (ignore physical groups)");
+                                       "Save all elements");
     y += BH;
     dialog->b[0]->type(FL_TOGGLE_BUTTON);
     dialog->b[1] =
@@ -1400,7 +1393,7 @@ int bdfFileDialog(const char *name)
     dialog->d->menu(tagmenu);
     dialog->d->align(FL_ALIGN_RIGHT);
     dialog->b = new Fl_Check_Button(WB, y, 2 * BBB + WB, BH,
-                                    "Save all (ignore physical groups)");
+                                    "Save all elements");
     y += BH;
     dialog->b->type(FL_TOGGLE_BUTTON);
     dialog->ok = new Fl_Return_Button(WB, y + WB, BBB, BH, "OK");
@@ -1477,7 +1470,7 @@ int genericMeshFileDialog(const char *name, const char *title, int format,
     dialog->d->menu(tagmenu);
     dialog->d->align(FL_ALIGN_RIGHT);
     dialog->b = new Fl_Check_Button(WB, y, 2 * BBB + WB, BH,
-                                    "Save all (ignore physical groups)");
+                                    "Save all elements");
     y += BH;
     dialog->b->type(FL_TOGGLE_BUTTON);
     dialog->ok = new Fl_Return_Button(WB, y + WB, BBB, BH, "OK");
