@@ -24,6 +24,7 @@
 #include "MPrism.h"
 #include "MPyramid.h"
 #include "discreteVertex.h"
+#include "Context.h"
 
 extern "C" {
 #include <med.h>
@@ -541,7 +542,18 @@ static void writeElementsMED(med_idt &fid, char *meshName,
 int GModel::writeMED(const std::string &name, bool saveAll,
                      double scalingFactor)
 {
+#if(MED_MAJOR_NUM == 3) && (MED_MINOR_NUM >= 3)
+  // MEDfileVersionOpen actually appeared in MED 3.2.1
+  med_int major = MED_MAJOR_NUM, minor = MED_MINOR_NUM, release = MED_RELEASE_NUM;
+  if(CTX::instance()->mesh.medFileMinorVersion >= 0){
+    minor = (int)CTX::instance()->mesh.medFileMinorVersion;
+    Msg::Info("Forcing MED file version to %d.%d", major, minor);
+  }
+  med_idt fid = MEDfileVersionOpen((char *)name.c_str(), MED_ACC_CREAT,
+                                   major, minor, release);
+#else
   med_idt fid = MEDouvrir((char *)name.c_str(), MED_CREATION);
+#endif
   if(fid < 0) {
     Msg::Error("Unable to open file '%s'", name.c_str());
     return 0;
