@@ -704,7 +704,7 @@ void writeMSHPeriodicNodes(FILE *fp, std::vector<GEntity *> &entities,
 int GModel::_writeMSH3(const std::string &name, double version, bool binary,
                        bool saveAll, bool saveParametric, double scalingFactor,
                        int elementStartNum, int saveSinglePartition,
-                       bool multipleView)
+                       bool append)
 {
   if(version < 3. || version >= 4.) {
     Msg::Error("Wrong MSH file version %g for MSH3 writer", version);
@@ -712,7 +712,7 @@ int GModel::_writeMSH3(const std::string &name, double version, bool binary,
   }
 
   FILE *fp;
-  if(multipleView)
+  if(append)
     fp = Fopen(name.c_str(), binary ? "ab" : "a");
   else
     fp = Fopen(name.c_str(), binary ? "wb" : "w");
