@@ -1,15 +1,13 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
-#include <math.h>
+#include <cmath>
 #include "drawContext.h"
 #include "GmshMessage.h"
 #include "GmshDefines.h"
 #include "GModel.h"
-#include "GFaceCompound.h"
-#include "GEdgeCompound.h"
 #include "MLine.h"
 #include "MTriangle.h"
 #include "MQuadrangle.h"
@@ -17,6 +15,7 @@
 #include "MHexahedron.h"
 #include "MPrism.h"
 #include "MPyramid.h"
+#include "MTrihedron.h"
 #include "MElementCut.h"
 #include "Context.h"
 #include "OS.h"
@@ -26,22 +25,27 @@
 #include "PView.h"
 #include "PViewData.h"
 
+#if defined(HAVE_FLTK)
+#include <FL/Fl.H>
+#include <FL/gl.h>
+#endif
+
 // from GModelVertexArrays
 extern unsigned int getColorByEntity(GEntity *e);
 extern bool isElementVisible(MElement *ele);
 
-template<class T>
+template <class T>
 static void drawElementLabels(drawContext *ctx, GEntity *e,
-                              std::vector<T*> &elements, int forceColor=0,
-                              unsigned int color=0)
+                              std::vector<T *> &elements, int forceColor = 0,
+                              unsigned int color = 0)
 {
   unsigned col = forceColor ? color : getColorByEntity(e);
-  glColor4ubv((GLubyte *) & col);
+  glColor4ubv((GLubyte *)&col);
 
   int labelStep = CTX::instance()->mesh.labelSampling;
   if(labelStep <= 0) labelStep = 1;
 
-  for(unsigned int i = 0; i < elements.size(); i++){
+  for(unsigned int i = 0; i < elements.size(); i++) {
     MElement *ele = elements[i];
     if(!isElementVisible(ele)) continue;
     if(i % labelStep == 0) {
@@ -51,7 +55,7 @@ static void drawElementLabels(drawContext *ctx, GEntity *e,
         sprintf(str, "(%g,%g,%g)", pc.x(), pc.y(), pc.z());
       else if(CTX::instance()->mesh.labelType == 3)
         sprintf(str, "%d", ele->getPartition());
-      else if(CTX::instance()->mesh.labelType == 2){
+      else if(CTX::instance()->mesh.labelType == 2) {
         int np = e->physicals.size();
         int p = np ? e->physicals[np - 1] : 0;
         sprintf(str, "%d", p);
@@ -60,17 +64,16 @@ static void drawElementLabels(drawContext *ctx, GEntity *e,
         sprintf(str, "%d", e->tag());
       else
         sprintf(str, "%d", ele->getNum());
-      glRasterPos3d(pc.x(), pc.y(), pc.z());
-      ctx->drawString(str);
+      ctx->drawString(str, pc.x(), pc.y(), pc.z());
     }
   }
 }
 
-template<class T>
-static void drawNormals(drawContext *ctx, std::vector<T*> &elements)
+template <class T>
+static void drawNormals(drawContext *ctx, std::vector<T *> &elements)
 {
-  glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.normals);
-  for(unsigned int i = 0; i < elements.size(); i++){
+  glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.normals);
+  for(unsigned int i = 0; i < elements.size(); i++) {
     MElement *ele = elements[i];
     if(!isElementVisible(ele)) continue;
     SVector3 n = ele->getFace(0).normal();
@@ -82,11 +85,11 @@ static void drawNormals(drawContext *ctx, std::vector<T*> &elements)
   }
 }
 
-template<class T>
-static void drawTangents(drawContext *ctx, std::vector<T*> &elements)
+template <class T>
+static void drawTangents(drawContext *ctx, std::vector<T *> &elements)
 {
-  glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.tangents);
-  for(unsigned int i = 0; i < elements.size(); i++){
+  glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.tangents);
+  for(unsigned int i = 0; i < elements.size(); i++) {
     MElement *ele = elements[i];
     if(!isElementVisible(ele)) continue;
     SVector3 t = ele->getEdge(0).tangent();
@@ -99,7 +102,7 @@ static void drawTangents(drawContext *ctx, std::vector<T*> &elements)
 }
 
 static void drawVertexLabel(drawContext *ctx, GEntity *e, MVertex *v,
-                            int partition=-1)
+                            int partition = -1)
 {
   if(!v->getVisibility()) return;
 
@@ -108,7 +111,7 @@ static void drawVertexLabel(drawContext *ctx, GEntity *e, MVertex *v,
   char str[256];
   if(CTX::instance()->mesh.labelType == 4)
     sprintf(str, "(%.16g,%.16g,%.16g)", v->x(), v->y(), v->z());
-  else if(CTX::instance()->mesh.labelType == 3){
+  else if(CTX::instance()->mesh.labelType == 3) {
     if(partition < 0)
       sprintf(str, "NA");
     else
@@ -123,69 +126,62 @@ static void drawVertexLabel(drawContext *ctx, GEntity *e, MVertex *v,
 
   if(CTX::instance()->mesh.colorCarousel == 0 ||
      CTX::instance()->mesh.volumesFaces ||
-     CTX::instance()->mesh.surfacesFaces){ // by element type
+     CTX::instance()->mesh.surfacesFaces) { // by element type
     if(v->getPolynomialOrder() > 1)
-      glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertexSup);
+      glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.nodeSup);
     else
-      glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertex);
+      glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.node);
   }
-  else{
+  else {
     unsigned int col = getColorByEntity(e);
-    glColor4ubv((GLubyte *) & col);
+    glColor4ubv((GLubyte *)&col);
   }
   double offset = (0.5 * CTX::instance()->mesh.pointSize +
-                   0.1 * CTX::instance()->glFontSize) * ctx->pixel_equiv_x;
-  glRasterPos3d(v->x() + offset / ctx->s[0],
-                v->y() + offset / ctx->s[1],
-                v->z() + offset / ctx->s[2]);
-  ctx->drawString(str);
+                   0.1 * CTX::instance()->glFontSize) *
+                  ctx->pixel_equiv_x;
+  ctx->drawString(str, v->x() + offset / ctx->s[0], v->y() + offset / ctx->s[1],
+                  v->z() + offset / ctx->s[2]);
 }
 
 static void drawVerticesPerEntity(drawContext *ctx, GEntity *e)
 {
-  //if(e->dim() == 2) {
-  //  if(e->cast2Edge()->getCompound()) {
-  //    if(e->cast2Edge()->getCompound()
-  //
-   // }
-  //}
   if(CTX::instance()->mesh.points) {
     if(CTX::instance()->mesh.pointType) {
-      for(unsigned int i = 0; i < e->mesh_vertices.size(); i++){
+      for(unsigned int i = 0; i < e->mesh_vertices.size(); i++) {
         MVertex *v = e->mesh_vertices[i];
         if(!v->getVisibility()) continue;
         if(CTX::instance()->mesh.colorCarousel == 0 ||
            CTX::instance()->mesh.volumesFaces ||
-           CTX::instance()->mesh.surfacesFaces){ // by element type
+           CTX::instance()->mesh.surfacesFaces) { // by element type
           if(v->getPolynomialOrder() > 1)
-            glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertexSup);
+            glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.nodeSup);
           else
-            glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertex);
+            glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.node);
         }
-        else{
+        else {
           unsigned int col = getColorByEntity(e);
-          glColor4ubv((GLubyte *) & col);
+          glColor4ubv((GLubyte *)&col);
         }
         ctx->drawSphere(CTX::instance()->mesh.pointSize, v->x(), v->y(), v->z(),
                         CTX::instance()->mesh.light);
       }
     }
-    else{
+    else {
       glBegin(GL_POINTS);
-      for(unsigned int i = 0; i < e->mesh_vertices.size(); i++){
+      for(unsigned int i = 0; i < e->mesh_vertices.size(); i++) {
         MVertex *v = e->mesh_vertices[i];
         if(!v->getVisibility()) continue;
         if(CTX::instance()->mesh.colorCarousel == 0 ||
            CTX::instance()->mesh.volumesFaces ||
-           CTX::instance()->mesh.surfacesFaces){ // by element type
+           CTX::instance()->mesh.surfacesFaces) { // by element type
           if(v->getPolynomialOrder() > 1)
-            glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertexSup);
+            glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.nodeSup);
           else
-            glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertex);
+            glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.node);
         }
-        else{
+        else {
           unsigned int col = getColorByEntity(e);
-          glColor4ubv((GLubyte *) & col);
+          glColor4ubv((GLubyte *)&col);
         }
         glVertex3d(v->x(), v->y(), v->z());
       }
@@ -200,34 +196,34 @@ static void drawVerticesPerEntity(drawContext *ctx, GEntity *e)
   }
 }
 
-template<class T>
+template <class T>
 static void drawVerticesPerElement(drawContext *ctx, GEntity *e,
-                                   std::vector<T*> &elements)
+                                   std::vector<T *> &elements)
 {
-  for(unsigned int i = 0; i < elements.size(); i++){
+  for(unsigned int i = 0; i < elements.size(); i++) {
     MElement *ele = elements[i];
-    for(int j = 0; j < ele->getNumVertices(); j++){
+    for(std::size_t j = 0; j < ele->getNumVertices(); j++) {
       MVertex *v = ele->getVertex(j);
       // FIXME isElementVisible() can be slow: we should also use a
       // vertex array for drawing vertices...
-      if(isElementVisible(ele) && v->getVisibility()){
+      if(isElementVisible(ele) && v->getVisibility()) {
         if(CTX::instance()->mesh.points) {
           if(CTX::instance()->mesh.colorCarousel == 0 ||
              CTX::instance()->mesh.volumesFaces ||
-             CTX::instance()->mesh.surfacesFaces){ // by element type
+             CTX::instance()->mesh.surfacesFaces) { // by element type
             if(v->getPolynomialOrder() > 1)
-              glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertexSup);
+              glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.nodeSup);
             else
-              glColor4ubv((GLubyte *) & CTX::instance()->color.mesh.vertex);
+              glColor4ubv((GLubyte *)&CTX::instance()->color.mesh.node);
           }
-          else{
+          else {
             unsigned int col = getColorByEntity(e);
-            glColor4ubv((GLubyte *) & col);
+            glColor4ubv((GLubyte *)&col);
           }
           if(CTX::instance()->mesh.pointType)
-            ctx->drawSphere(CTX::instance()->mesh.pointSize, v->x(), v->y(), v->z(),
-                            CTX::instance()->mesh.light);
-          else{
+            ctx->drawSphere(CTX::instance()->mesh.pointSize, v->x(), v->y(),
+                            v->z(), CTX::instance()->mesh.light);
+          else {
             glBegin(GL_POINTS);
             glVertex3d(v->x(), v->y(), v->z());
             glEnd();
@@ -240,35 +236,35 @@ static void drawVerticesPerElement(drawContext *ctx, GEntity *e,
   }
 }
 
-template<class T>
-static void drawBarycentricDual(std::vector<T*> &elements)
+template <class T> static void drawBarycentricDual(std::vector<T *> &elements)
 {
-  glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+  glColor4ubv((GLubyte *)&CTX::instance()->color.fg);
   glEnable(GL_LINE_STIPPLE);
   glLineStipple(1, 0x0F0F);
   gl2psEnable(GL2PS_LINE_STIPPLE);
   glBegin(GL_LINES);
-  for(unsigned int i = 0; i < elements.size(); i++){
+  for(unsigned int i = 0; i < elements.size(); i++) {
     MElement *ele = elements[i];
     if(!isElementVisible(ele)) continue;
     SPoint3 pc = ele->barycenter();
-    if(ele->getDim() == 2){
-      for(int j = 0; j < ele->getNumEdges(); j++){
+    if(ele->getDim() == 2) {
+      for(int j = 0; j < ele->getNumEdges(); j++) {
         MEdge e = ele->getEdge(j);
         SPoint3 p = e.barycenter();
         glVertex3d(pc.x(), pc.y(), pc.z());
         glVertex3d(p.x(), p.y(), p.z());
       }
     }
-    else if(ele->getDim() == 3){
-      for(int j = 0; j < ele->getNumFaces(); j++){
+    else if(ele->getDim() == 3) {
+      for(int j = 0; j < ele->getNumFaces(); j++) {
         MFace f = ele->getFace(j);
         SPoint3 p = f.barycenter();
         glVertex3d(pc.x(), pc.y(), pc.z());
         glVertex3d(p.x(), p.y(), p.z());
-        for(int k = 0; k < f.getNumVertices(); k++){
+        for(std::size_t k = 0; k < f.getNumVertices(); k++) {
           MEdge e(f.getVertex(k), (k == f.getNumVertices() - 1) ?
-                  f.getVertex(0) : f.getVertex(k + 1));
+                                    f.getVertex(0) :
+                                    f.getVertex(k + 1));
           SPoint3 pe = e.barycenter();
           glVertex3d(p.x(), p.y(), p.z());
           glVertex3d(pe.x(), pe.y(), pe.z());
@@ -281,20 +277,19 @@ static void drawBarycentricDual(std::vector<T*> &elements)
   gl2psDisable(GL2PS_LINE_STIPPLE);
 }
 
-template<class T>
-static void drawVoronoiDual(std::vector<T*> &elements)
+template <class T> static void drawVoronoiDual(std::vector<T *> &elements)
 {
-  glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+  glColor4ubv((GLubyte *)&CTX::instance()->color.fg);
   glEnable(GL_LINE_STIPPLE);
   glLineStipple(1, 0x0F0F);
   gl2psEnable(GL2PS_LINE_STIPPLE);
   glBegin(GL_LINES);
-  for(unsigned int i = 0; i < elements.size(); i++){
+  for(unsigned int i = 0; i < elements.size(); i++) {
     T *ele = elements[i];
     if(!isElementVisible(ele)) continue;
     SPoint3 pc = ele->circumcenter();
-    if(ele->getDim() == 2){
-      for(int j = 0; j < ele->getNumEdges(); j++){
+    if(ele->getDim() == 2) {
+      for(int j = 0; j < ele->getNumEdges(); j++) {
         MEdge e = ele->getEdge(j);
         SVector3 p2p1(e.getVertex(1)->x() - e.getVertex(0)->x(),
                       e.getVertex(1)->y() - e.getVertex(0)->y(),
@@ -302,23 +297,25 @@ static void drawVoronoiDual(std::vector<T*> &elements)
         SVector3 pcp1(pc.x() - e.getVertex(0)->x(),
                       pc.y() - e.getVertex(0)->y(),
                       pc.z() - e.getVertex(0)->z());
-        double alpha = dot(pcp1,p2p1) / dot(p2p1,p2p1);
-        SPoint3 p((1 - alpha)*e.getVertex(0)->x() + alpha * e.getVertex(1)->x(),
-                  (1 - alpha)*e.getVertex(0)->y() + alpha * e.getVertex(1)->y(),
-                  (1 - alpha)*e.getVertex(0)->z() + alpha * e.getVertex(1)->z());
+        double alpha = dot(pcp1, p2p1) / dot(p2p1, p2p1);
+        SPoint3 p(
+          (1 - alpha) * e.getVertex(0)->x() + alpha * e.getVertex(1)->x(),
+          (1 - alpha) * e.getVertex(0)->y() + alpha * e.getVertex(1)->y(),
+          (1 - alpha) * e.getVertex(0)->z() + alpha * e.getVertex(1)->z());
         glVertex3d(pc.x(), pc.y(), pc.z());
         glVertex3d(p.x(), p.y(), p.z());
       }
     }
-    else if(ele->getDim() == 3){
-      for(int j = 0; j < ele->getNumFaces(); j++){
+    else if(ele->getDim() == 3) {
+      for(int j = 0; j < ele->getNumFaces(); j++) {
         MFace f = ele->getFace(j);
         SPoint3 p = f.barycenter();
         glVertex3d(pc.x(), pc.y(), pc.z());
         glVertex3d(p.x(), p.y(), p.z());
-        for(int k = 0; k < f.getNumVertices(); k++){
+        for(std::size_t k = 0; k < f.getNumVertices(); k++) {
           MEdge e(f.getVertex(k), (k == f.getNumVertices() - 1) ?
-                  f.getVertex(0) : f.getVertex(k + 1));
+                                    f.getVertex(0) :
+                                    f.getVertex(k + 1));
           SPoint3 pe = e.barycenter();
           glVertex3d(p.x(), p.y(), p.z());
           glVertex3d(pe.x(), pe.y(), pe.z());
@@ -333,18 +330,21 @@ static void drawVoronoiDual(std::vector<T*> &elements)
 
 // Routine for drawing the vertex arrays
 
-static void drawArrays(drawContext *ctx, GEntity *e, VertexArray *va, GLint type,
-                       bool useNormalArray, int forceColor=0, unsigned int color=0)
+static void drawArrays(drawContext *ctx, GEntity *e, VertexArray *va,
+                       GLint type, bool useNormalArray, int forceColor = 0,
+                       unsigned int color = 0)
 {
   if(!va || !va->getNumVertices()) return;
 
   // If we want to be enable picking of individual elements we need to
   // draw each one separately
-  bool select = (ctx->render_mode == drawContext::GMSH_SELECT &&
-                 CTX::instance()->pickElements && e->model() == GModel::current());
+  bool select =
+    (ctx->render_mode == drawContext::GMSH_SELECT &&
+     CTX::instance()->pickElements && e->model() == GModel::current());
   if(select) {
-    if(va->getNumElementPointers() == va->getNumVertices()){
-      for(int i = 0; i < va->getNumVertices(); i += va->getNumVerticesPerElement()){
+    if(va->getNumElementPointers() == va->getNumVertices()) {
+      for(int i = 0; i < va->getNumVertices();
+          i += va->getNumVerticesPerElement()) {
         glPushName(va->getNumVerticesPerElement());
         glPushName(i);
         glBegin(type);
@@ -361,28 +361,28 @@ static void drawArrays(drawContext *ctx, GEntity *e, VertexArray *va, GLint type
   glVertexPointer(3, GL_FLOAT, 0, va->getVertexArray());
   glEnableClientState(GL_VERTEX_ARRAY);
 
-  if(useNormalArray){
+  if(useNormalArray) {
     glEnable(GL_LIGHTING);
-    glNormalPointer(GL_BYTE, 0, va->getNormalArray());
+    glNormalPointer(NORMAL_GLTYPE, 0, va->getNormalArray());
     glEnableClientState(GL_NORMAL_ARRAY);
   }
   else
     glDisableClientState(GL_NORMAL_ARRAY);
 
-  if(forceColor){
+  if(forceColor) {
     glDisableClientState(GL_COLOR_ARRAY);
-    glColor4ubv((GLubyte *) & color);
+    glColor4ubv((GLubyte *)&color);
   }
   else if(CTX::instance()->pickElements ||
           (!e->getSelection() && (CTX::instance()->mesh.colorCarousel == 0 ||
-                                  CTX::instance()->mesh.colorCarousel == 3))){
+                                  CTX::instance()->mesh.colorCarousel == 3))) {
     glColorPointer(4, GL_UNSIGNED_BYTE, 0, va->getColorArray());
     glEnableClientState(GL_COLOR_ARRAY);
   }
-  else{
+  else {
     glDisableClientState(GL_COLOR_ARRAY);
     color = getColorByEntity(e);
-    glColor4ubv((GLubyte *) & color);
+    glColor4ubv((GLubyte *)&color);
   }
 
   if(va->getNumVerticesPerElement() > 2 && CTX::instance()->polygonOffset)
@@ -401,11 +401,12 @@ static void drawArrays(drawContext *ctx, GEntity *e, VertexArray *va, GLint type
 // GVertex drawing routines
 
 class drawMeshGVertex {
- private:
+private:
   drawContext *_ctx;
- public:
-  drawMeshGVertex(drawContext *ctx) : _ctx(ctx){}
-  void operator () (GVertex *v)
+
+public:
+  drawMeshGVertex(drawContext *ctx) : _ctx(ctx) {}
+  void operator()(GVertex *v)
   {
     if(!v->getVisibility()) return;
 
@@ -416,6 +417,8 @@ class drawMeshGVertex {
       glPushName(v->tag());
     }
 
+    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+
     if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum)
       drawVerticesPerEntity(_ctx, v);
 
@@ -429,18 +432,15 @@ class drawMeshGVertex {
 // GEdge drawing routines
 
 class drawMeshGEdge {
- private:
+private:
   drawContext *_ctx;
- public:
-  drawMeshGEdge(drawContext *ctx) : _ctx(ctx){}
-  void operator () (GEdge *e)
+
+public:
+  drawMeshGEdge(drawContext *ctx) : _ctx(ctx) {}
+  void operator()(GEdge *e)
   {
     if(!e->getVisibility()) {
-      if(e->getCompound()) {
-        if(!e->getCompound()->getVisibility()) return;
-      }
-      else
-        return;
+      return;
     }
 
     bool select = (_ctx->render_mode == drawContext::GMSH_SELECT &&
@@ -450,21 +450,21 @@ class drawMeshGEdge {
       glPushName(e->tag());
     }
 
+    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+
     if(CTX::instance()->mesh.lines)
       drawArrays(_ctx, e, e->va_lines, GL_LINES, false);
 
-    if(CTX::instance()->mesh.linesNum)
-      drawElementLabels(_ctx, e, e->lines);
+    if(CTX::instance()->mesh.linesNum) drawElementLabels(_ctx, e, e->lines);
 
-    if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum){
+    if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum) {
       if(e->getAllElementsVisible())
         drawVerticesPerEntity(_ctx, e);
       else
         drawVerticesPerElement(_ctx, e, e->lines);
     }
 
-    if(CTX::instance()->mesh.tangents)
-      drawTangents(_ctx, e->lines);
+    if(CTX::instance()->mesh.tangents) drawTangents(_ctx, e->lines);
 
     if(select) {
       glPopName();
@@ -476,18 +476,15 @@ class drawMeshGEdge {
 // GFace drawing routines
 
 class drawMeshGFace {
- private:
+private:
   drawContext *_ctx;
- public:
-  drawMeshGFace(drawContext *ctx) : _ctx(ctx){}
-  void operator () (GFace *f)
+
+public:
+  drawMeshGFace(drawContext *ctx) : _ctx(ctx) {}
+  void operator()(GFace *f)
   {
     if(!f->getVisibility()) {
-      if(f->getCompound()) {
-        if(!f->getCompound()->getVisibility()) return;
-      }
-      else
-        return;
+      return;
     }
 
     bool select = (_ctx->render_mode == drawContext::GMSH_SELECT &&
@@ -497,26 +494,37 @@ class drawMeshGFace {
       glPushName(f->tag());
     }
 
+    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+
     drawArrays(_ctx, f, f->va_lines, GL_LINES,
                CTX::instance()->mesh.light && CTX::instance()->mesh.lightLines,
-               CTX::instance()->mesh.surfacesFaces, CTX::instance()->color.mesh.line);
-    drawArrays(_ctx, f, f->va_triangles, GL_TRIANGLES, CTX::instance()->mesh.light);
+               CTX::instance()->mesh.surfacesFaces,
+               CTX::instance()->color.mesh.line);
+
+    if(CTX::instance()->mesh.lightTwoSide)
+      glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+
+    drawArrays(_ctx, f, f->va_triangles, GL_TRIANGLES,
+               CTX::instance()->mesh.light);
 
     if(CTX::instance()->mesh.surfacesNum) {
       if(CTX::instance()->mesh.triangles)
-        drawElementLabels(_ctx, f, f->triangles, CTX::instance()->mesh.surfacesFaces,
+        drawElementLabels(_ctx, f, f->triangles,
+                          CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
       if(CTX::instance()->mesh.quadrangles)
-        drawElementLabels(_ctx, f, f->quadrangles, CTX::instance()->mesh.surfacesFaces,
+        drawElementLabels(_ctx, f, f->quadrangles,
+                          CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
-      drawElementLabels(_ctx, f, f->polygons, CTX::instance()->mesh.surfacesFaces,
+      drawElementLabels(_ctx, f, f->polygons,
+                        CTX::instance()->mesh.surfacesFaces,
                         CTX::instance()->color.mesh.line);
     }
 
-    if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum){
+    if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum) {
       if(f->getAllElementsVisible())
         drawVerticesPerEntity(_ctx, f);
-      else{
+      else {
         if(CTX::instance()->mesh.triangles)
           drawVerticesPerElement(_ctx, f, f->triangles);
         if(CTX::instance()->mesh.quadrangles)
@@ -550,11 +558,12 @@ class drawMeshGFace {
 // GRegion drawing routines
 
 class drawMeshGRegion {
- private:
+private:
   drawContext *_ctx;
- public:
-  drawMeshGRegion(drawContext *ctx) : _ctx(ctx){}
-  void operator () (GRegion *r)
+
+public:
+  drawMeshGRegion(drawContext *ctx) : _ctx(ctx) {}
+  void operator()(GRegion *r)
   {
     if(!r->getVisibility()) return;
 
@@ -565,41 +574,65 @@ class drawMeshGRegion {
       glPushName(r->tag());
     }
 
-    drawArrays(_ctx, r, r->va_lines, GL_LINES, CTX::instance()->mesh.light &&
-               CTX::instance()->mesh.lightLines, CTX::instance()->mesh.volumesFaces,
-               CTX::instance()->color.mesh.line);
-    drawArrays(_ctx, r, r->va_triangles, GL_TRIANGLES, CTX::instance()->mesh.light);
+    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+
+    drawArrays(
+      _ctx, r, r->va_lines, GL_LINES,
+      CTX::instance()->mesh.light && (CTX::instance()->mesh.lightLines > 1),
+      CTX::instance()->mesh.volumesFaces, CTX::instance()->color.mesh.line);
+
+    if(CTX::instance()->mesh.lightTwoSide)
+      glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+
+    drawArrays(_ctx, r, r->va_triangles, GL_TRIANGLES,
+               CTX::instance()->mesh.light);
 
     if(CTX::instance()->mesh.volumesNum) {
       if(CTX::instance()->mesh.tetrahedra)
-        drawElementLabels(_ctx, r, r->tetrahedra, CTX::instance()->mesh.volumesFaces ||
-                          CTX::instance()->mesh.surfacesFaces,
+        drawElementLabels(_ctx, r, r->tetrahedra,
+                          CTX::instance()->mesh.volumesFaces ||
+                            CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
       if(CTX::instance()->mesh.hexahedra)
-        drawElementLabels(_ctx, r, r->hexahedra, CTX::instance()->mesh.volumesFaces ||
-                          CTX::instance()->mesh.surfacesFaces,
+        drawElementLabels(_ctx, r, r->hexahedra,
+                          CTX::instance()->mesh.volumesFaces ||
+                            CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
       if(CTX::instance()->mesh.prisms)
-        drawElementLabels(_ctx, r, r->prisms, CTX::instance()->mesh.volumesFaces ||
-                          CTX::instance()->mesh.surfacesFaces,
+        drawElementLabels(_ctx, r, r->prisms,
+                          CTX::instance()->mesh.volumesFaces ||
+                            CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
       if(CTX::instance()->mesh.pyramids)
-        drawElementLabels(_ctx, r, r->pyramids, CTX::instance()->mesh.volumesFaces ||
-                          CTX::instance()->mesh.surfacesFaces,
+        drawElementLabels(_ctx, r, r->pyramids,
+                          CTX::instance()->mesh.volumesFaces ||
+                            CTX::instance()->mesh.surfacesFaces,
                           CTX::instance()->color.mesh.line);
-      drawElementLabels(_ctx, r, r->polyhedra, CTX::instance()->mesh.volumesFaces ||
-                        CTX::instance()->mesh.surfacesFaces,
+      if(CTX::instance()->mesh.trihedra)
+        drawElementLabels(_ctx, r, r->trihedra,
+                          CTX::instance()->mesh.volumesFaces ||
+                            CTX::instance()->mesh.surfacesFaces,
+                          CTX::instance()->color.mesh.line);
+      drawElementLabels(_ctx, r, r->polyhedra,
+                        CTX::instance()->mesh.volumesFaces ||
+                          CTX::instance()->mesh.surfacesFaces,
                         CTX::instance()->color.mesh.line);
     }
 
-    if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum){
+    if(CTX::instance()->mesh.points || CTX::instance()->mesh.pointsNum) {
       if(r->getAllElementsVisible())
         drawVerticesPerEntity(_ctx, r);
-      else{
-        if(CTX::instance()->mesh.tetrahedra) drawVerticesPerElement(_ctx, r, r->tetrahedra);
-        if(CTX::instance()->mesh.hexahedra) drawVerticesPerElement(_ctx, r, r->hexahedra);
-        if(CTX::instance()->mesh.prisms) drawVerticesPerElement(_ctx, r, r->prisms);
-        if(CTX::instance()->mesh.pyramids) drawVerticesPerElement(_ctx, r, r->pyramids);
+      else {
+        if(CTX::instance()->mesh.tetrahedra)
+          drawVerticesPerElement(_ctx, r, r->tetrahedra);
+        if(CTX::instance()->mesh.hexahedra)
+          drawVerticesPerElement(_ctx, r, r->hexahedra);
+        if(CTX::instance()->mesh.prisms)
+          drawVerticesPerElement(_ctx, r, r->prisms);
+        if(CTX::instance()->mesh.pyramids)
+          drawVerticesPerElement(_ctx, r, r->pyramids);
+        if(CTX::instance()->mesh.trihedra)
+          drawVerticesPerElement(_ctx, r, r->trihedra);
         drawVerticesPerElement(_ctx, r, r->polyhedra);
       }
     }
@@ -609,6 +642,7 @@ class drawMeshGRegion {
       if(CTX::instance()->mesh.hexahedra) drawBarycentricDual(r->hexahedra);
       if(CTX::instance()->mesh.prisms) drawBarycentricDual(r->prisms);
       if(CTX::instance()->mesh.pyramids) drawBarycentricDual(r->pyramids);
+      if(CTX::instance()->mesh.trihedra) drawBarycentricDual(r->trihedra);
       drawBarycentricDual(r->polyhedra);
     }
 
@@ -650,7 +684,7 @@ void drawContext::drawMesh()
   // make sure to flag any model-dependent post-processing view as
   // changed if the underlying mesh has, before resetting the changed
   // flag
-  if(CTX::instance()->mesh.changed){
+  if(CTX::instance()->mesh.changed) {
     for(unsigned int i = 0; i < GModel::list.size(); i++)
       for(unsigned int j = 0; j < PView::list.size(); j++)
         if(PView::list[j]->getData()->hasModel(GModel::list[i]))
@@ -665,12 +699,7 @@ void drawContext::drawMesh()
   gl2psLineWidth((float)(CTX::instance()->mesh.lineWidth *
                          CTX::instance()->print.epsLineWidthFactor));
 
-  if(CTX::instance()->mesh.lightTwoSide)
-    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-  else
-    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
-
-  if(!CTX::instance()->clipWholeElements){
+  if(!CTX::instance()->clipWholeElements) {
     for(int i = 0; i < 6; i++)
       if(CTX::instance()->mesh.clip & (1 << i))
         glEnable((GLenum)(GL_CLIP_PLANE0 + i));
@@ -678,16 +707,21 @@ void drawContext::drawMesh()
         glDisable((GLenum)(GL_CLIP_PLANE0 + i));
   }
 
-  for(unsigned int i = 0; i < GModel::list.size(); i++){
+  for(unsigned int i = 0; i < GModel::list.size(); i++) {
     GModel *m = GModel::list[i];
-    m->fillVertexArrays();
-    if(m->getVisibility() && isVisible(m)){
+    bool changed = m->fillVertexArrays();
+    if(changed) Msg::Debug("mesh vertex arrays have changed");
+#if defined(HAVE_FLTK) && defined(__APPLE__)
+    // FIXME: resetting texture pile fixes bug with recent MacOS versions
+    if(changed) gl_texture_pile_height(gl_texture_pile_height());
+#endif
+    if(m->getVisibility() && isVisible(m)) {
       int status = m->getMeshStatus();
       if(status >= 0)
         std::for_each(m->firstVertex(), m->lastVertex(), drawMeshGVertex(this));
       if(status >= 1)
         std::for_each(m->firstEdge(), m->lastEdge(), drawMeshGEdge(this));
-      if(status >= 2){
+      if(status >= 2) {
         beginFakeTransparency();
         std::for_each(m->firstFace(), m->lastFace(), drawMeshGFace(this));
         endFakeTransparency();
@@ -699,6 +733,5 @@ void drawContext::drawMesh()
 
   CTX::instance()->mesh.changed = 0;
 
-  for(int i = 0; i < 6; i++)
-    glDisable((GLenum)(GL_CLIP_PLANE0 + i));
+  for(int i = 0; i < 6; i++) glDisable((GLenum)(GL_CLIP_PLANE0 + i));
 }
