@@ -1,9 +1,9 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
-#include <math.h>
+#include <cmath>
 #include <algorithm>
 #include "GmshConfig.h"
 #include "GmshDefines.h"
@@ -33,20 +33,25 @@ static void drawArrays(drawContext *ctx, PView *p, VertexArray *va, GLint type,
   if(CTX::instance()->polygonOffset || opt->showElement)
     glEnable(GL_POLYGON_OFFSET_FILL);
 
-  if(type == GL_POINTS && opt->pointType > 0){
-    for(int i = 0; i < va->getNumVertices(); i++){
+  if(type == GL_POINTS && opt->pointType > 0) {
+    for(int i = 0; i < va->getNumVertices(); i++) {
       float *p = va->getVertexArray(3 * i);
       glColor4ubv((GLubyte *)va->getColorArray(4 * i));
       double f = 1.;
-      if(opt->pointType > 1){
+      if(opt->pointType > 1) {
+#if defined(HAVE_VISUDEV)
+        f = *va->getNormalArray(3 * i);
+#else
         char *n = va->getNormalArray(3 * i);
         f = char2float(*n);
+#endif
       }
-      if(opt->pointType == 2){
+      if(opt->pointType == 2) {
         int s = (int)(opt->pointSize * f);
-        if(s){
+        if(s) {
           glPointSize((float)s);
-          gl2psPointSize((float)(s * CTX::instance()->print.epsPointSizeFactor));
+          gl2psPointSize(
+            (float)(s * CTX::instance()->print.epsPointSizeFactor));
           glBegin(GL_POINTS);
           glVertex3d(p[0], p[1], p[2]);
           glEnd();
@@ -56,41 +61,54 @@ static void drawArrays(drawContext *ctx, PView *p, VertexArray *va, GLint type,
         ctx->drawSphere(opt->pointSize * f, p[0], p[1], p[2], opt->light);
     }
   }
-  else if(type == GL_LINES && opt->lineType > 0){
-    for(int i = 0; i < va->getNumVertices(); i += 2){
+  else if(type == GL_LINES && opt->lineType > 0) {
+    for(int i = 0; i < va->getNumVertices(); i += 2) {
       float *p0 = va->getVertexArray(3 * i);
       float *p1 = va->getVertexArray(3 * (i + 1));
-      double x[2] = {p0[0], p1[0]}, y[2] = {p0[1], p1[1]}, z[2] = {p0[2], p1[2]};
+      double x[2] = {p0[0], p1[0]}, y[2] = {p0[1], p1[1]},
+             z[2] = {p0[2], p1[2]};
       glColor4ubv((GLubyte *)va->getColorArray(4 * i));
-      if(opt->lineType == 2){
+      if(opt->lineType == 2) {
+#if defined(HAVE_VISUDEV)
+        double v0 = *va->getNormalArray(3 * i);
+        double v1 = *va->getNormalArray(3 * (i + 1));
+#else
         char *n0 = va->getNormalArray(3 * i);
         char *n1 = va->getNormalArray(3 * (i + 1));
         double v0 = char2float(*n0), v1 = char2float(*n1);
-        ctx->drawTaperedCylinder(opt->lineWidth, v0, v1, 0., 1., x, y, z, opt->light);
+#endif
+        ctx->drawTaperedCylinder(opt->lineWidth, v0, v1, 0., 1., x, y, z,
+                                 opt->light);
       }
-      else if (opt->lineType == 1)
+      else if(opt->lineType == 1)
         ctx->drawCylinder(opt->lineWidth, x, y, z, opt->light);
       else { // 2D (for now) MNT diagrams for frames
-	float l = sqrt ((p0[0] - p1[0]) * (p0[0] - p1[0]) +
-                        (p0[1] - p1[1]) * (p0[1] - p1[1]) +
-                        (p0[2] - p1[2]) * (p0[2] - p1[2]) );
+        float l = sqrt((p0[0] - p1[0]) * (p0[0] - p1[0]) +
+                       (p0[1] - p1[1]) * (p0[1] - p1[1]) +
+                       (p0[2] - p1[2]) * (p0[2] - p1[2]));
+#if defined(HAVE_VISUDEV)
+        double v0 = *va->getNormalArray(3 * i);
+        double v1 = *va->getNormalArray(3 * (i + 1));
+#else
         char *n0 = va->getNormalArray(3 * i);
         char *n1 = va->getNormalArray(3 * (i + 1));
         double v0 = char2float(*n0), v1 = char2float(*n1);
-	float dir [3] = {(p1[0] - p0[0]) / l , (p1[1] - p0[1]) / l , (p1[2] - p0[2]) / l};
-	printf("%g %g %g %g %g %g\n", v0, v1, p0[0], p0[1], p1[0], p1[1]);
-	ctx->drawVector(1, 0,
-			p0[0] - dir[1] * v0 , p0[1] + dir[0] * v0 , 0.0,
-			p1[0] - dir[1] * v1 , p1[1] + dir[0] * v1 , 0.0,opt->light);
+#endif
+        float dir[3] = {(p1[0] - p0[0]) / l, (p1[1] - p0[1]) / l,
+                        (p1[2] - p0[2]) / l};
+        printf("%g %g %g %g %g %g\n", v0, v1, p0[0], p0[1], p1[0], p1[1]);
+        ctx->drawVector(1, 0, p0[0] - dir[1] * v0, p0[1] + dir[0] * v0, 0.0,
+                        p1[0] - dir[1] * v1, p1[1] + dir[0] * v1, 0.0,
+                        opt->light);
       }
     }
   }
-  else{
+  else {
     glVertexPointer(3, GL_FLOAT, 0, va->getVertexArray());
     glEnableClientState(GL_VERTEX_ARRAY);
-    if(useNormalArray){
+    if(useNormalArray) {
       glEnable(GL_LIGHTING);
-      glNormalPointer(GL_BYTE, 0, va->getNormalArray());
+      glNormalPointer(NORMAL_GLTYPE, 0, va->getNormalArray());
       glEnableClientState(GL_NORMAL_ARRAY);
     }
     else
@@ -117,18 +135,19 @@ static void drawEllipseArray(drawContext *ctx, PView *p, VertexArray *va)
     float *s = va->getVertexArray(3 * i);
     float vv[3][3];
     double lmax = opt->tmpMax;
-    double scale = (opt->arrowSizeMax - opt->arrowSizeMin) *ctx->pixel_equiv_x / ctx->s[0]/2;
-    double lmin = opt->arrowSizeMin * ctx->pixel_equiv_x / ctx->s[0]/2;
-    for (int j = 0; j < 3; j++) {
+    double scale = (opt->arrowSizeMax - opt->arrowSizeMin) *
+                   ctx->pixel_equiv_x / ctx->s[0] / 2;
+    double lmin = opt->arrowSizeMin * ctx->pixel_equiv_x / ctx->s[0] / 2;
+    for(int j = 0; j < 3; j++) {
       float *v = va->getVertexArray(3 * (i + j + 1));
-      double l = sqrt (v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
-      double l2 = std::min(1., l/lmax);
-      for (int k = 0; k < 3; k++) {
-        vv[j][k] = v[k]/l*(scale*l2 + lmin);
+      double l = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
+      double l2 = std::min(1., l / lmax);
+      for(int k = 0; k < 3; k++) {
+        vv[j][k] = v[k] / l * (scale * l2 + lmin);
       }
     }
     glColor4ubv((GLubyte *)va->getColorArray(4 * i));
-    if (opt->tensorType == PViewOptions::Ellipsoid)
+    if(opt->tensorType == PViewOptions::Ellipsoid)
       ctx->drawEllipsoid(s[0], s[1], s[2], vv[0], vv[1], vv[2], opt->light);
     else
       ctx->drawEllipse(s[0], s[1], s[2], vv[0], vv[1], opt->light);
@@ -141,19 +160,20 @@ static void drawVectorArray(drawContext *ctx, PView *p, VertexArray *va)
 
   PViewOptions *opt = p->getOptions();
 
-  for(int i = 0; i < va->getNumVertices(); i += 2){
+  for(int i = 0; i < va->getNumVertices(); i += 2) {
     float *s = va->getVertexArray(3 * i);
     float *v = va->getVertexArray(3 * (i + 1));
     glColor4ubv((GLubyte *)va->getColorArray(4 * i));
-    double l = sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
+    double vv[3] = {v[0], v[1], v[2]};
+    double l = sqrt(vv[0] * vv[0] + vv[1] * vv[1] + vv[2] * vv[2]);
     double lmax = opt->tmpMax;
-    if((l || opt->vectorType == 6) && lmax){
+    if((l || opt->vectorType == 6) && lmax) {
       double scale = (opt->arrowSizeMax - opt->arrowSizeMin) / lmax;
       // log scaling
-      if(opt->scaleType == PViewOptions::Logarithmic &&
-         opt->tmpMin > 0 && opt->tmpMax > opt->tmpMin && l != opt->tmpMin){
+      if(opt->scaleType == PViewOptions::Logarithmic && opt->tmpMin > 0 &&
+         opt->tmpMax > opt->tmpMin && l != opt->tmpMin) {
         scale = (opt->arrowSizeMax - opt->arrowSizeMin) / l *
-          log10(l / opt->tmpMin) / log10(opt->tmpMax / opt->tmpMin);
+                log10(l / opt->tmpMin) / log10(opt->tmpMax / opt->tmpMin);
       }
       if(opt->arrowSizeMin && l) scale += opt->arrowSizeMin / l;
       double px = scale * v[0];
@@ -161,18 +181,24 @@ static void drawVectorArray(drawContext *ctx, PView *p, VertexArray *va)
       double pz = scale * v[2];
       // only draw vectors larger than 1 pixel on screen, except when
       // drawing "comet" glyphs
-      if(opt->vectorType == 6 || fabs(px) > 1. || fabs(py) > 1. || fabs(pz) > 1.){
+      if(opt->vectorType == 6 || fabs(px) > 1. || fabs(py) > 1. ||
+         fabs(pz) > 1.) {
         double d = ctx->pixel_equiv_x / ctx->s[0];
         double dx = px * d, dy = py * d, dz = pz * d;
         double x = s[0], y = s[1], z = s[2];
-        if(opt->centerGlyphs == 2){
-          x -= dx; y -= dy; z -= dz;
+        if(opt->centerGlyphs == 2) {
+          x -= dx;
+          y -= dy;
+          z -= dz;
         }
-        else if(opt->centerGlyphs == 1){
-          x -= 0.5 * dx; y -= 0.5 * dy; z -= 0.5 * dz;
+        else if(opt->centerGlyphs == 1) {
+          x -= 0.5 * dx;
+          y -= 0.5 * dy;
+          z -= 0.5 * dz;
         }
-        ctx->drawVector(opt->vectorType, opt->intervalsType != PViewOptions::Iso,
-                        x, y, z, dx, dy, dz, opt->light);
+        ctx->drawVector(opt->vectorType,
+                        opt->intervalsType != PViewOptions::Iso, x, y, z, dx,
+                        dy, dz, opt->light);
       }
     }
   }
@@ -184,7 +210,7 @@ static std::string stringValue(int numComp, double d[9], double norm,
   char label[100];
   if(numComp == 1)
     sprintf(label, format, d[0]);
-  else if(numComp == 3){
+  else if(numComp == 3) {
     char str[3][32];
     sprintf(str[0], format, d[0]);
     sprintf(str[1], format, d[1]);
@@ -196,48 +222,54 @@ static std::string stringValue(int numComp, double d[9], double norm,
   return std::string(label);
 }
 
-static void drawNumberGlyphs(drawContext *ctx, PView *p, int numNodes, int numComp,
-                             double **xyz, double **val)
+static void drawNumberGlyphs(drawContext *ctx, PView *p, int numNodes,
+                             int numComp, double **xyz, double **val)
 {
   PViewOptions *opt = p->getOptions();
   double d[9] = {0., 0., 0., 0., 0., 0., 0., 0., 0.};
 
   double vmin = opt->tmpMin, vmax = opt->tmpMax;
 
-  if(opt->glyphLocation == PViewOptions::COG){
+  if(opt->glyphLocation == PViewOptions::COG) {
     SPoint3 pc(0., 0., 0.);
-    for(int i = 0; i < numNodes; i++){
+    for(int i = 0; i < numNodes; i++) {
       pc += SPoint3(xyz[i][0], xyz[i][1], xyz[i][2]);
       for(int j = 0; j < numComp; j++) d[j] += val[i][j];
     }
     pc /= (double)numNodes;
     for(int j = 0; j < numComp; j++) d[j] /= (double)numNodes;
     double v = ComputeScalarRep(numComp, d);
-    if(v >= vmin && v <= vmax){
+    if(v >= vmin && v <= vmax) {
       unsigned int col = opt->getColor(v, vmin, vmax, false, opt->nbIso);
-      glColor4ubv((GLubyte *) & col);
-      glRasterPos3d(pc.x(), pc.y(), pc.z());
+      glColor4ubv((GLubyte *)&col);
       if(opt->centerGlyphs == 2)
-        ctx->drawStringRight(stringValue(numComp, d, v, opt->format.c_str()));
+        ctx->drawStringRight(stringValue(numComp, d, v, opt->format.c_str()),
+                             pc.x(), pc.y(), pc.z());
       else if(opt->centerGlyphs == 1)
-        ctx->drawStringCenter(stringValue(numComp, d, v, opt->format.c_str()));
+        ctx->drawStringCenter(stringValue(numComp, d, v, opt->format.c_str()),
+                              pc.x(), pc.y(), pc.z());
       else
-        ctx->drawString(stringValue(numComp, d, v, opt->format.c_str()));
+        ctx->drawString(stringValue(numComp, d, v, opt->format.c_str()), pc.x(),
+                        pc.y(), pc.z());
     }
   }
-  else if(opt->glyphLocation == PViewOptions::Vertex){
-    for(int i = 0; i < numNodes; i++){
+  else if(opt->glyphLocation == PViewOptions::Vertex) {
+    for(int i = 0; i < numNodes; i++) {
       double v = ComputeScalarRep(numComp, val[i]);
-      if(v >= vmin && v <= vmax){
+      if(v >= vmin && v <= vmax) {
         unsigned int col = opt->getColor(v, vmin, vmax, false, opt->nbIso);
-        glColor4ubv((GLubyte *) & col);
-        glRasterPos3d(xyz[i][0], xyz[i][1], xyz[i][2]);
+        glColor4ubv((GLubyte *)&col);
         if(opt->centerGlyphs == 2)
-          ctx->drawStringRight(stringValue(numComp, val[i], v, opt->format.c_str()));
+          ctx->drawStringRight(
+            stringValue(numComp, val[i], v, opt->format.c_str()), xyz[i][0],
+            xyz[i][1], xyz[i][2]);
         else if(opt->centerGlyphs == 1)
-          ctx->drawStringCenter(stringValue(numComp, val[i], v, opt->format.c_str()));
+          ctx->drawStringCenter(
+            stringValue(numComp, val[i], v, opt->format.c_str()), xyz[i][0],
+            xyz[i][1], xyz[i][2]);
         else
-          ctx->drawString(stringValue(numComp, val[i], v, opt->format.c_str()));
+          ctx->drawString(stringValue(numComp, val[i], v, opt->format.c_str()),
+                          xyz[i][0], xyz[i][1], xyz[i][2]);
       }
     }
   }
@@ -253,16 +285,18 @@ static void drawNormalVectorGlyphs(drawContext *ctx, PView *p, int numNodes,
     pc += SPoint3(xyz[i][0], xyz[i][1], xyz[i][2]);
   pc /= (double)numNodes;
 
-  SVector3 t1(xyz[1][0] - xyz[0][0], xyz[1][1] - xyz[0][1], xyz[1][2] - xyz[0][2]);
-  SVector3 t2(xyz[2][0] - xyz[0][0], xyz[2][1] - xyz[0][1], xyz[2][2] - xyz[0][2]);
+  SVector3 t1(xyz[1][0] - xyz[0][0], xyz[1][1] - xyz[0][1],
+              xyz[1][2] - xyz[0][2]);
+  SVector3 t2(xyz[2][0] - xyz[0][0], xyz[2][1] - xyz[0][1],
+              xyz[2][2] - xyz[0][2]);
   SVector3 n = crossprod(t1, t2);
   n.normalize();
 
   for(int i = 0; i < 3; i++)
     n[i] *= opt->normals * ctx->pixel_equiv_x / ctx->s[i];
-  glColor4ubv((GLubyte *) & opt->color.normals);
-  ctx->drawVector(CTX::instance()->vectorType, 0, pc[0], pc[1], pc[2], n[0], n[1], n[2],
-                  opt->light);
+  glColor4ubv((GLubyte *)&opt->color.normals);
+  ctx->drawVector(CTX::instance()->vectorType, 0, pc[0], pc[1], pc[2], n[0],
+                  n[1], n[2], opt->light);
 }
 
 static void drawTangentVectorGlyphs(drawContext *ctx, PView *p, int numNodes,
@@ -277,9 +311,9 @@ static void drawTangentVectorGlyphs(drawContext *ctx, PView *p, int numNodes,
   t.normalize();
   for(int i = 0; i < 3; i++)
     t[i] *= opt->tangents * ctx->pixel_equiv_x / ctx->s[i];
-  glColor4ubv((GLubyte *) & opt->color.tangents);
-  ctx->drawVector(CTX::instance()->vectorType, 0, pc[0], pc[1], pc[2], t[0], t[1], t[2],
-                  opt->light);
+  glColor4ubv((GLubyte *)&opt->color.tangents);
+  ctx->drawVector(CTX::instance()->vectorType, 0, pc[0], pc[1], pc[2], t[0],
+                  t[1], t[2], opt->light);
 }
 
 static void drawGlyphs(drawContext *ctx, PView *p)
@@ -290,14 +324,15 @@ static void drawGlyphs(drawContext *ctx, PView *p)
   PViewData *data = p->getData(true);
   PViewOptions *opt = p->getOptions();
 
-  if(!opt->normals && !opt->tangents && opt->intervalsType != PViewOptions::Numeric)
+  if(!opt->normals && !opt->tangents &&
+     opt->intervalsType != PViewOptions::Numeric)
     return;
 
   Msg::Debug("drawing extra glyphs (this is slow...)");
 
   // speedup drawing of textured fonts on cocoa mac version
-#if defined(HAVE_FLTK) && defined(__APPLE__) && defined(HAVE_64BIT_SIZE_T)
-  if(opt->intervalsType == PViewOptions::Numeric){
+#if defined(HAVE_FLTK) && defined(__APPLE__)
+  if(opt->intervalsType == PViewOptions::Numeric) {
     int numStrings = 0;
     for(int ent = 0; ent < data->getNumEntities(opt->timeStep); ent++)
       numStrings += data->getNumElements(opt->timeStep, ent);
@@ -306,53 +341,57 @@ static void drawGlyphs(drawContext *ctx, PView *p)
   }
 #endif
 
-  //double xyz[PVIEW_NMAX][3], val[PVIEW_NMAX][9];
+  // double xyz[PVIEW_NMAX][3], val[PVIEW_NMAX][9];
   int NMAX = PVIEW_NMAX;
-  double **xyz = new double*[NMAX];
-  double **val = new double*[NMAX];
-  for(int i = 0; i < NMAX; i++){
+  double **xyz = new double *[NMAX];
+  double **val = new double *[NMAX];
+  for(int i = 0; i < NMAX; i++) {
     xyz[i] = new double[3];
     val[i] = new double[9];
   }
-  for(int ent = 0; ent < data->getNumEntities(opt->timeStep); ent++){
+  for(int ent = 0; ent < data->getNumEntities(opt->timeStep); ent++) {
     if(data->skipEntity(opt->timeStep, ent)) continue;
-    for(int i = 0; i < data->getNumElements(opt->timeStep, ent); i++){
-      if(data->skipElement(opt->timeStep, ent, i, true, opt->sampling)) continue;
+    for(int i = 0; i < data->getNumElements(opt->timeStep, ent); i++) {
+      if(data->skipElement(opt->timeStep, ent, i, true, opt->sampling))
+        continue;
       int type = data->getType(opt->timeStep, ent, i);
       if(opt->skipElement(type)) continue;
       int dim = data->getDimension(opt->timeStep, ent, i);
       int numComp = data->getNumComponents(opt->timeStep, ent, i);
       int numNodes = data->getNumNodes(opt->timeStep, ent, i);
-      if(numNodes > NMAX){
-        if(type == TYPE_POLYG || type == TYPE_POLYH){
-          for(int j = 0; j < NMAX; j++){
-            delete [] xyz[i];
-            delete [] val[i];
+      if(numNodes > NMAX) {
+        if(type == TYPE_POLYG || type == TYPE_POLYH) {
+          for(int j = 0; j < NMAX; j++) {
+            delete[] xyz[i];
+            delete[] val[i];
           }
-          delete [] xyz;
-          delete [] val;
+          delete[] xyz;
+          delete[] val;
           NMAX = numNodes;
-          xyz = new double*[NMAX];
-          val = new double*[NMAX];
-          for(int j = 0; j < NMAX; j++){
+          xyz = new double *[NMAX];
+          val = new double *[NMAX];
+          for(int j = 0; j < NMAX; j++) {
             xyz[j] = new double[3];
             val[j] = new double[9];
           }
         }
-        else{
-          if(numNodesError != numNodes){
+        else {
+          if(numNodesError != numNodes) {
             numNodesError = numNodes;
-            Msg::Warning("Fields with %d nodes per element cannot be displayed: "
-                         "either force the field type or select 'Adapt visualization "
-                         "grid' if the field is high-order", numNodes);
+            Msg::Warning(
+              "Fields with %d nodes per element cannot be displayed: "
+              "either force the field type or select 'Adapt visualization "
+              "grid' if the field is high-order",
+              numNodes);
           }
           continue;
         }
       }
-      for(int j = 0; j < numNodes; j++){
-        data->getNode(opt->timeStep, ent, i, j, xyz[j][0], xyz[j][1], xyz[j][2]);
-        if(opt->forceNumComponents){
-          for(int k = 0; k < opt->forceNumComponents; k++){
+      for(int j = 0; j < numNodes; j++) {
+        data->getNode(opt->timeStep, ent, i, j, xyz[j][0], xyz[j][1],
+                      xyz[j][2]);
+        if(opt->forceNumComponents) {
+          for(int k = 0; k < opt->forceNumComponents; k++) {
             int comp = opt->componentMap[k];
             if(comp >= 0 && comp < numComp)
               data->getValue(opt->timeStep, ent, i, j, comp, val[j][k]);
@@ -375,19 +414,19 @@ static void drawGlyphs(drawContext *ctx, PView *p)
         drawTangentVectorGlyphs(ctx, p, numNodes, xyz, val);
     }
   }
-  for(int j = 0; j < NMAX; j++){
-    delete [] xyz[j];
-    delete [] val[j];
+  for(int j = 0; j < NMAX; j++) {
+    delete[] xyz[j];
+    delete[] val[j];
   }
-  delete [] xyz;
-  delete [] val;
+  delete[] xyz;
+  delete[] val;
 }
 
 static bool eyeChanged(drawContext *ctx, PView *p)
 {
   double zeye = 100 * CTX::instance()->lc;
   SPoint3 tmp(ctx->rot[2] * zeye, ctx->rot[6] * zeye, ctx->rot[10] * zeye);
-  if(tmp.distance(p->getEye()) > 1.e-3){
+  if(tmp.distance(p->getEye()) > 1.e-3) {
     p->setEye(tmp);
     return true;
   }
@@ -395,11 +434,12 @@ static bool eyeChanged(drawContext *ctx, PView *p)
 }
 
 class drawPView {
- private:
+private:
   drawContext *_ctx;
- public:
+
+public:
   drawPView(drawContext *ctx) : _ctx(ctx) {}
-  void operator () (PView *p)
+  void operator()(PView *p)
   {
     // use adaptive data if available
     PViewData *data = p->getData(true);
@@ -409,33 +449,35 @@ class drawPView {
     if(!opt->visible || opt->type != PViewOptions::Plot3D) return;
     if(!_ctx->isVisible(p)) return;
 
+    if(_ctx->render_mode == drawContext::GMSH_SELECT) {
+      glPushName(5);
+      glPushName(p->getIndex());
+    }
+
     glPointSize((float)opt->pointSize);
-    gl2psPointSize((float)(opt->pointSize *
-                           CTX::instance()->print.epsPointSizeFactor));
+    gl2psPointSize(
+      (float)(opt->pointSize * CTX::instance()->print.epsPointSizeFactor));
 
     glLineWidth((float)opt->lineWidth);
-    gl2psLineWidth((float)(opt->lineWidth *
-                           CTX::instance()->print.epsLineWidthFactor));
+    gl2psLineWidth(
+      (float)(opt->lineWidth * CTX::instance()->print.epsLineWidthFactor));
 
-    if(opt->lightTwoSide)
-      glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
-    else
-      glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
-
-    if(opt->axes && opt->type == PViewOptions::Plot3D){
-      glColor4ubv((GLubyte *) & opt->color.axes);
+    if(opt->axes && opt->type == PViewOptions::Plot3D) {
+      glColor4ubv((GLubyte *)&opt->color.axes);
       glLineWidth((float)CTX::instance()->lineWidth);
       gl2psLineWidth((float)(CTX::instance()->lineWidth *
                              CTX::instance()->print.epsLineWidthFactor));
       if(!opt->axesAutoPosition)
-        _ctx->drawAxes(opt->axes, opt->axesTics, opt->axesFormat, opt->axesLabel,
-                       opt->axesPosition, opt->axesMikado, opt->axesPosition);
+        _ctx->drawAxes(opt->axes, opt->axesTics, opt->axesFormat,
+                       opt->axesLabel, opt->axesPosition, opt->axesMikado,
+                       opt->axesPosition);
       else if(!opt->tmpBBox.empty())
-        _ctx->drawAxes(opt->axes, opt->axesTics, opt->axesFormat, opt->axesLabel,
-                       opt->tmpBBox, opt->axesMikado, opt->tmpBBox);
+        _ctx->drawAxes(opt->axes, opt->axesTics, opt->axesFormat,
+                       opt->axesLabel, opt->tmpBBox, opt->axesMikado,
+                       opt->tmpBBox);
     }
 
-    if(!CTX::instance()->clipWholeElements){
+    if(!CTX::instance()->clipWholeElements) {
       for(int i = 0; i < 6; i++)
         if(opt->clip & (1 << i))
           glEnable((GLenum)(GL_CLIP_PLANE0 + i));
@@ -443,8 +485,8 @@ class drawPView {
           glDisable((GLenum)(GL_CLIP_PLANE0 + i));
     }
 
-    if(CTX::instance()->alpha && ColorTable_IsAlpha(&opt->colorTable)){
-      if(opt->fakeTransparency){
+    if(CTX::instance()->alpha && ColorTable_IsAlpha(&opt->colorTable)) {
+      if(opt->fakeTransparency) {
         // simple additive blending "a la xpost":
         glBlendFunc(GL_SRC_ALPHA, GL_ONE); // glBlendEquation(GL_FUNC_ADD);
         // maximum intensity projection "a la volsuite":
@@ -452,28 +494,29 @@ class drawPView {
         glEnable(GL_BLEND);
         glDisable(GL_DEPTH_TEST);
       }
-      else{
+      else {
         // real translucent blending (requires back-to-front traversal)
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         // glBlendEquation(GL_FUNC_ADD);
         glEnable(GL_BLEND);
         if(p->va_triangles && p->va_triangles->getNumVertices() &&
-           eyeChanged(_ctx, p)){
+           eyeChanged(_ctx, p)) {
           Msg::Debug("Sorting View[%d] for transparency", p->getIndex());
-          p->va_triangles->sort(p->getEye().x(), p->getEye().y(), p->getEye().z());
+          p->va_triangles->sort(p->getEye().x(), p->getEye().y(),
+                                p->getEye().z());
         }
       }
     }
 
-    if(opt->rangeType == PViewOptions::Custom){
-       opt->tmpMin = opt->customMin;
-       opt->tmpMax = opt->customMax;
+    if(opt->rangeType == PViewOptions::Custom) {
+      opt->tmpMin = opt->customMin;
+      opt->tmpMax = opt->customMax;
     }
-    else if(opt->rangeType == PViewOptions::PerTimeStep){
+    else if(opt->rangeType == PViewOptions::PerTimeStep) {
       opt->tmpMin = data->getMin(opt->timeStep);
       opt->tmpMax = data->getMax(opt->timeStep);
     }
-    else{
+    else {
       // FIXME: this is not perfect for multi-step adaptive views, as
       // we don't have the correct min/max info for the other steps
       opt->tmpMin = data->getMin();
@@ -481,8 +524,13 @@ class drawPView {
     }
 
     // draw all the vertex arrays
+    glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);
+
     drawArrays(_ctx, p, p->va_points, GL_POINTS, false);
     drawArrays(_ctx, p, p->va_lines, GL_LINES, opt->light && opt->lightLines);
+
+    if(opt->lightTwoSide) glLightModelf(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+
     drawArrays(_ctx, p, p->va_triangles, GL_TRIANGLES, opt->light);
 
     // draw the "pseudo" vertex arrays for vectors
@@ -495,33 +543,37 @@ class drawPView {
     drawGlyphs(_ctx, p);
 
     // draw the 3D strings
-    if(opt->drawStrings){
-      glColor4ubv((GLubyte *) & opt->color.text3d);
-      for(int i = 0; i < data->getNumStrings3D(); i++){
+    if(opt->drawStrings) {
+      glColor4ubv((GLubyte *)&opt->color.text3d);
+      for(int i = 0; i < data->getNumStrings3D(); i++) {
         double x, y, z, style;
         std::string str;
         data->getString3D(i, opt->timeStep, str, x, y, z, style);
-        glRasterPos3d(x, y, z);
-        _ctx->drawString(str, style);
+        _ctx->drawString(str, x, y, z, style);
       }
     }
 
-    if(CTX::instance()->alpha){
+    if(CTX::instance()->alpha) {
       glDisable(GL_BLEND);
       glEnable(GL_DEPTH_TEST);
     }
 
-    for(int i = 0; i < 6; i++)
-      glDisable((GLenum)(GL_CLIP_PLANE0 + i));
+    for(int i = 0; i < 6; i++) glDisable((GLenum)(GL_CLIP_PLANE0 + i));
+
+    if(_ctx->render_mode == drawContext::GMSH_SELECT) {
+      glPopName();
+      glPopName();
+    }
   }
 };
 
 class drawPViewBoundingBox {
- private:
+private:
   drawContext *_ctx;
- public:
+
+public:
   drawPViewBoundingBox(drawContext *ctx) : _ctx(ctx) {}
-  void operator () (PView *p)
+  void operator()(PView *p)
   {
     PViewData *data = p->getData();
     PViewOptions *opt = p->getOptions();
@@ -531,22 +583,21 @@ class drawPViewBoundingBox {
     SBoundingBox3d bb = data->getBoundingBox(opt->timeStep);
     if(bb.empty()) return;
 
-    glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
+    glColor4ubv((GLubyte *)&CTX::instance()->color.fg);
     glLineWidth((float)CTX::instance()->lineWidth);
     gl2psLineWidth((float)(CTX::instance()->lineWidth *
                            CTX::instance()->print.epsLineWidthFactor));
 
-    _ctx->drawBox(bb.min().x(), bb.min().y(), bb.min().z(),
-                  bb.max().x(), bb.max().y(), bb.max().z());
+    _ctx->drawBox(bb.min().x(), bb.min().y(), bb.min().z(), bb.max().x(),
+                  bb.max().y(), bb.max().z());
     glColor3d(1., 0., 0.);
     for(int i = 0; i < 6; i++)
       if(opt->clip & (1 << i))
-        _ctx->drawPlaneInBoundingBox(bb.min().x(), bb.min().y(), bb.min().z(),
-                                     bb.max().x(), bb.max().y(), bb.max().z(),
-                                     CTX::instance()->clipPlane[i][0],
-                                     CTX::instance()->clipPlane[i][1],
-                                     CTX::instance()->clipPlane[i][2],
-                                     CTX::instance()->clipPlane[i][3]);
+        _ctx->drawPlaneInBoundingBox(
+          bb.min().x(), bb.min().y(), bb.min().z(), bb.max().x(), bb.max().y(),
+          bb.max().z(), CTX::instance()->clipPlane[i][0],
+          CTX::instance()->clipPlane[i][1], CTX::instance()->clipPlane[i][2],
+          CTX::instance()->clipPlane[i][3]);
   }
 };
 
@@ -558,11 +609,19 @@ void drawContext::drawPost()
   if(PView::list.empty()) return;
 
   if(CTX::instance()->drawBBox || !CTX::instance()->post.draw)
-    std::for_each(PView::list.begin(), PView::list.end(), drawPViewBoundingBox(this));
+    std::for_each(PView::list.begin(), PView::list.end(),
+                  drawPViewBoundingBox(this));
 
   if(!CTX::instance()->post.draw) return;
 
-  for(unsigned int i = 0; i < PView::list.size(); i++)
-    PView::list[i]->fillVertexArrays();
+  for(unsigned int i = 0; i < PView::list.size(); i++) {
+    bool changed = PView::list[i]->fillVertexArrays();
+    if(changed) Msg::Debug("post-pro vertex arrays have changed");
+#if defined(HAVE_FLTK) && defined(__APPLE__)
+    // FIXME: resetting texture pile fixes bug with recent MacOS versions
+    if(changed) gl_texture_pile_height(gl_texture_pile_height());
+#endif
+  }
+
   std::for_each(PView::list.begin(), PView::list.end(), drawPView(this));
 }
