@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _FULL_MATRIX_H_
 #define _FULL_MATRIX_H_
@@ -10,6 +10,7 @@
 #include "GmshMessage.h"
 #include <cmath>
 #include <cstdio>
+#include <complex>
 
 template <class scalar> class fullMatrix;
 
@@ -38,28 +39,26 @@ template <class scalar> class fullMatrix;
 */
 
 // An abstract interface for vectors of scalar
-template <class scalar>
-class fullVector
-{
- private:
-  int _r;          // size of the vector
-  scalar *_data;   // pointer on the first element
+template <class scalar> class fullVector {
+private:
+  int _r; // size of the vector
+  scalar *_data; // pointer on the first element
   bool _own_data;
   friend class fullMatrix<scalar>;
 
- public:
+public:
   // constructor and destructor
   /**
      Instantiates a zero size fullVector.
   */
-  fullVector(void) : _r(0),_data(0),_own_data(1) {}
+  fullVector(void) : _r(0), _data(0), _own_data(1) {}
 
   /**
      @param r A positive integer.
 
      Instantiates a fullVector of size r filled with zeros.
   */
-  fullVector(int r) : _r(r),_own_data(1)
+  fullVector(int r) : _r(r), _own_data(1)
   {
     _data = new scalar[_r];
     setAll(scalar(0.));
@@ -84,7 +83,7 @@ class fullVector
 
      Instantiates a fullVector, which is a copy (and not a proxy) of other.
   */
-  fullVector(const fullVector<scalar> &other) : _r(other._r),_own_data(1)
+  fullVector(const fullVector<scalar> &other) : _r(other._r), _own_data(1)
   {
     _data = new scalar[_r];
     for(int i = 0; i < _r; ++i) _data[i] = other._data[i];
@@ -95,40 +94,40 @@ class fullVector
   */
   ~fullVector()
   {
-    if(_own_data && _data) delete [] _data;
+    if(_own_data && _data) delete[] _data;
   }
 
   // get information (size, value)
   /**
      @return Returns the size of this fullVector
   */
-  inline int size()                  const { return _r; }
+  inline int size() const { return _r; }
 
   /**
      @return Returns a const pointer to this fullVector data.@n
      This pointer will point to the following memory segment:
      [(*this)(0), (*this)(1), ..., (*this)(this->size() - 1)].
   */
-  inline const scalar * getDataPtr() const { return _data; }
+  inline const scalar *getDataPtr() const { return _data; }
 
   /**
      @return Returns a  pointer to this fullVector data.@n
      This pointer will point to the following memory segment:
      [(*this)(0), (*this)(1), ..., (*this)(this->size() - 1)].
   */
-  inline scalar * getDataPtr() { return _data; }
+  inline scalar *getDataPtr() { return _data; }
 
   /**
      @param i A vector index between 0 and size() - 1.
      @returns Returns the ith scalar of this fullVector.
   */
-  inline scalar operator () (int i)  const { return _data[i]; }
+  inline scalar operator()(int i) const { return _data[i]; }
 
   /**
      @param i A vector index between 0 and size() - 1.
      @returns Returns a reference to the ith scalar of this fullVector.
   */
-  inline scalar & operator () (int i)      { return _data[i]; }
+  inline scalar &operator()(int i) { return _data[i]; }
 
   // copy
   /**
@@ -137,11 +136,11 @@ class fullVector
      The right hand side fullVector will become a copy of other,
      and will loose its previous data.
   */
-  fullVector<scalar>& operator= (const fullVector<scalar> &other)
+  fullVector<scalar> &operator=(const fullVector<scalar> &other)
   {
-    if (this != &other) {
-      if ((!resize(other.size(), false) && _r > 2*other.size())) {
-        if (_data) delete[] _data;
+    if(this != &other) {
+      if((!resize(other.size(), false) && _r > 2 * other.size())) {
+        if(_data) delete[] _data;
         _r = other.size();
         _data = new scalar[_r];
       }
@@ -163,12 +162,12 @@ class fullVector
 
      The rth value of this fullVector is set to v.
   */
-  inline void set(int r, scalar v){
-    #ifdef _DEBUG
-    if (r >= _r || r < 0)
-      Msg::Fatal("invalid index to access fullVector : %i (size = %i)",
-                 r, _r);
-    #endif
+  inline void set(int r, scalar v)
+  {
+#ifdef _DEBUG
+    if(r >= _r || r < 0)
+      Msg::Fatal("invalid index to access fullVector : %i (size = %i)", r, _r);
+#endif
     (*this)(r) = v;
   }
 
@@ -176,12 +175,7 @@ class fullVector
   /**
      @return Returns the @f$ L^2 @f$ norm of this fullVector.
   */
-  inline scalar norm() const
-  {
-    scalar n = 0.;
-    for(int i = 0; i < _r; ++i) n += _data[i] * _data[i];
-    return sqrt(n);
-  }
+  inline scalar norm() const; // Specialised in this file
 
   /**
      @param r A positive integer;
@@ -203,18 +197,16 @@ class fullVector
   */
   bool resize(int r, bool resetValue = true)
   {
-    if (_r < r || !_own_data) {
-      if (_own_data && _data) delete[] _data;
+    if(_r < r || !_own_data) {
+      if(_own_data && _data) delete[] _data;
       _r = r;
       _data = new scalar[_r];
       _own_data = true;
-      if(resetValue)
-        setAll(scalar(0.));
+      if(resetValue) setAll(scalar(0.));
       return true;
     }
     _r = r;
-    if(resetValue)
-      setAll(scalar(0.));
+    if(resetValue) setAll(scalar(0.));
     return false;
   }
 
@@ -230,7 +222,7 @@ class fullVector
   */
   void setAsProxy(const fullVector<scalar> &original, int r_start, int r)
   {
-    if(_own_data && _data) delete [] _data;
+    if(_own_data && _data) delete[] _data;
     _own_data = false;
     _r = r;
     _data = original._data + r_start;
@@ -247,7 +239,7 @@ class fullVector
   */
   void setAsProxy(const fullMatrix<scalar> &original, int c)
   {
-    if(_own_data && _data) delete [] _data;
+    if(_own_data && _data) delete[] _data;
     _own_data = false;
     _r = original._r;
     _data = original._data + c * _r;
@@ -262,7 +254,7 @@ class fullVector
   {
     if(s == scalar(0.))
       for(int i = 0; i < _r; ++i) _data[i] = scalar(0.);
-    else if (s == -1.)
+    else if(s == -1.)
       for(int i = 0; i < _r; ++i) _data[i] = -_data[i];
     else
       for(int i = 0; i < _r; ++i) _data[i] *= s;
@@ -275,7 +267,7 @@ class fullVector
   */
   inline void setAll(const scalar &m)
   {
-    for(int i = 0; i < _r; i++) set(i,m);
+    for(int i = 0; i < _r; i++) set(i, m);
   }
 
   /**
@@ -303,7 +295,7 @@ class fullVector
 
      @return Returns the scalar product of this fullVector with the other.
   */
-  inline scalar operator *(const fullVector<scalar> &other)
+  inline scalar operator*(const fullVector<scalar> &other)
   {
     scalar s = 0.;
     for(int i = 0; i < _r; ++i) s += _data[i] * other._data[i];
@@ -324,10 +316,10 @@ class fullVector
 
      x.size() must be greater or equal to @f$ N @f$.
   */
-  void axpy(const fullVector<scalar> &x, scalar alpha=1.)
+  void axpy(const fullVector<scalar> &x, scalar alpha = 1.)
 #if !defined(HAVE_BLAS)
   {
-    for (int i = 0; i < _r; i++) _data[i] += alpha * x._data[i];
+    for(int i = 0; i < _r; i++) _data[i] += alpha * x._data[i];
   }
 #endif
   ;
@@ -347,7 +339,7 @@ class fullVector
   */
   void multTByT(const fullVector<scalar> &x)
   {
-    for (int i = 0; i < _r; i++) _data[i] *= x._data[i];
+    for(int i = 0; i < _r; i++) _data[i] *= x._data[i];
   }
 
   // printing and file treatment
@@ -359,11 +351,11 @@ class fullVector
 
      This string starts by name.
   */
-  void print(const char *name="") const
+  void print(const char *name = "") const
   {
-    printf("double %s[%d]=\n", name,size());
+    printf("double %s[%d]=\n", name, size());
     printf("{  ");
-    for(int I = 0; I < size(); I++){
+    for(int I = 0; I < size(); I++) {
       printf("%12.5E ", (*this)(I));
     }
     printf("};\n");
@@ -375,10 +367,7 @@ class fullVector
      Writes a binary representation of this fullVector
      into f.
   */
-  void binarySave (FILE *f) const
-  {
-    fwrite (_data, sizeof(scalar), _r, f);
-  }
+  void binarySave(FILE *f) const { fwrite(_data, sizeof(scalar), _r, f); }
 
   /**
      @param f A pointer to a FILE stream containing a fullVector.
@@ -388,23 +377,40 @@ class fullVector
 
      @see fullVector::binarySave
   */
-  void binaryLoad (FILE *f)
+  void binaryLoad(FILE *f)
   {
-    if(fread (_data, sizeof(scalar), _r, f) != (size_t)_r) return;
+    if(fread(_data, sizeof(scalar), _r, f) != (size_t)_r) return;
   }
+
+  bool getOwnData() const { return _own_data; };
+  void setOwnData(bool ownData) { _own_data = ownData; };
 };
 
-// An abstract interface for dense matrix of scalar
-template <class scalar>
-class fullMatrix
+// Specialisation of fullVector<scalar>::norm() const
+template <> inline double fullVector<double>::norm() const
 {
- private:
+  double n = 0.;
+  for(int i = 0; i < _r; ++i) n += _data[i] * _data[i];
+  return sqrt(n);
+}
+template <>
+inline std::complex<double> fullVector<std::complex<double> >::norm() const
+{
+  double n = 0.;
+  for(int i = 0; i < _r; ++i)
+    n += _data[i].real() * _data[i].real() + _data[i].imag() * _data[i].imag();
+  return std::complex<double>(sqrt(n), 0.);
+}
+
+// An abstract interface for dense matrix of scalar
+template <class scalar> class fullMatrix {
+private:
   bool _own_data; // should data be freed on delete ?
   int _r, _c; // size of the matrix
   scalar *_data; // pointer on the first element
   friend class fullVector<scalar>;
 
- public:
+public:
   // constructor and destructor
   fullMatrix(scalar *original, int r, int c)
   {
@@ -424,10 +430,9 @@ class fullMatrix
   {
     _data = new scalar[_r * _c];
     _own_data = true;
-    if (init0)
-      setAll(scalar(0.));
+    if(init0) setAll(scalar(0.));
   }
-  fullMatrix(int r, int c, double *data)
+  fullMatrix(int r, int c, scalar *data)
     : _r(r), _c(c), _data(data), _own_data(false)
   {
     setAll(scalar(0.));
@@ -435,13 +440,13 @@ class fullMatrix
   fullMatrix(const fullMatrix<scalar> &other) : _r(other._r), _c(other._c)
   {
     _data = new scalar[_r * _c];
-    _own_data=true;
+    _own_data = true;
     for(int i = 0; i < _r * _c; ++i) _data[i] = other._data[i];
   }
-  fullMatrix() : _own_data(false),_r(0), _c(0), _data(0) {}
+  fullMatrix() : _own_data(false), _r(0), _c(0), _data(0) {}
   ~fullMatrix()
   {
-    if(_data && _own_data) delete [] _data;
+    if(_data && _own_data) delete[] _data;
   }
 
   // get information (size, value)
@@ -449,66 +454,66 @@ class fullMatrix
   inline int size2() const { return _c; }
   inline scalar get(int r, int c) const
   {
-    #ifdef _DEBUG
-    if (r >= _r || r < 0 || c >= _c || c < 0)
-      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)",
-                 r, c, _r, _c);
-    #endif
+#ifdef _DEBUG
+    if(r >= _r || r < 0 || c >= _c || c < 0)
+      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)", r,
+                 c, _r, _c);
+#endif
     return (*this)(r, c);
   }
 
-  inline const scalar * getDataPtr() const { return _data; }
-  inline scalar * getDataPtr() { return _data; }
+  inline const scalar *getDataPtr() const { return _data; }
+  inline scalar *getDataPtr() { return _data; }
 
   // operations
-  inline void set(int r, int c, scalar v){
-    #ifdef _DEBUG
-    if (r >= _r || r < 0 || c >= _c || c < 0)
-      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)",
-                 r, c, _r, _c);
-    #endif
+  inline void set(int r, int c, scalar v)
+  {
+#ifdef _DEBUG
+    if(r >= _r || r < 0 || c >= _c || c < 0)
+      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)", r,
+                 c, _r, _c);
+#endif
     (*this)(r, c) = v;
   }
   inline scalar norm() const
   {
     scalar n = 0.;
     for(int i = 0; i < _r; ++i)
-      for(int j = 0; j < _c; ++j)
-        n += (*this)(i, j) * (*this)(i, j);
+      for(int j = 0; j < _c; ++j) n += (*this)(i, j) * (*this)(i, j);
     return sqrt(n);
   }
-  bool resize(int r, int c, bool resetValue = true) // data will be owned (same as constructor)
+  bool
+  resize(int r, int c,
+         bool resetValue = true) // data will be owned (same as constructor)
   {
-    if ((r * c > _r * _c) || !_own_data) {
-      if (_own_data && _data) delete[] _data;
+    if((r * c > _r * _c) || !_own_data) {
+      if(_own_data && _data) delete[] _data;
       _r = r;
       _c = c;
       _data = new scalar[_r * _c];
       _own_data = true;
-      if(resetValue)
-        setAll(scalar(0.));
+      if(resetValue) setAll(scalar(0.));
       return true;
     }
     _r = r;
     _c = c;
-    if(resetValue)
-      setAll(scalar(0.));
+    if(resetValue) setAll(scalar(0.));
     return false; // no reallocation
   }
-  void reshape(int nbRows, int nbColumns){
-    if (nbRows == -1 && nbColumns != -1)
-      nbRows = _r * _c / nbColumns;
-    if (nbRows != -1 && nbColumns == -1)
-      nbColumns = _r * _c / nbRows;
-    if (nbRows*nbColumns != size1()*size2())
-      Msg::Error("Invalid reshape, total number of entries must be equal");
+  void reshape(int nbRows, int nbColumns)
+  {
+    if(nbRows == -1 && nbColumns != -1) nbRows = _r * _c / nbColumns;
+    if(nbRows != -1 && nbColumns == -1) nbColumns = _r * _c / nbRows;
+    if(nbRows * nbColumns != size1() * size2())
+      Msg::Error("Invalid reshape, total number of entries must be equal (new "
+                 "%i x %i != old %i x %i)",
+                 nbRows, nbColumns, size1(), size2());
     _r = nbRows;
     _c = nbColumns;
   }
   void setAsProxy(const fullMatrix<scalar> &original)
   {
-    if(_data && _own_data)
-      delete [] _data;
+    if(_data && _own_data) delete[] _data;
     _c = original._c;
     _r = original._r;
     _own_data = false;
@@ -516,49 +521,47 @@ class fullMatrix
   }
   void setAsProxy(const fullMatrix<scalar> &original, int c_start, int c)
   {
-    if(_data && _own_data)
-      delete [] _data;
+    if(_data && _own_data) delete[] _data;
     _c = c;
     _r = original._r;
     _own_data = false;
     _data = original._data + c_start * _r;
   }
-  void setAsProxy(double *data, int r, int c)
+  void setAsProxy(scalar *data, int r, int c)
   {
-    if(_data && _own_data)
-      delete [] _data;
+    if(_data && _own_data) delete[] _data;
     _c = c;
     _r = r;
     _own_data = false;
     _data = data;
   }
-  fullMatrix<scalar> & operator = (const fullMatrix<scalar> &other)
+  fullMatrix<scalar> &operator=(const fullMatrix<scalar> &other)
   {
     copy(other);
     return *this;
   }
-  void operator += (const fullMatrix<scalar> &other)
+  void operator+=(const fullMatrix<scalar> &other)
   {
-    if(_r != other._r || _c!= other._c)
+    if(_r != other._r || _c != other._c)
       Msg::Error("sum matrices of different sizes\n");
     for(int i = 0; i < _r * _c; ++i) _data[i] += other._data[i];
   }
-  inline scalar operator () (int i, int j) const
+  inline scalar operator()(int i, int j) const
   {
-    #ifdef _DEBUG
-    if (i >= _r || i < 0 || j >= _c || j < 0)
-      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)",
-                 i, j, _r, _c);
-    #endif
+#ifdef _DEBUG
+    if(i >= _r || i < 0 || j >= _c || j < 0)
+      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)", i,
+                 j, _r, _c);
+#endif
     return _data[i + _r * j];
   }
-  inline scalar & operator () (int i, int j)
+  inline scalar &operator()(int i, int j)
   {
-    #ifdef _DEBUG
-    if (i >= _r || i < 0 || j >= _c || j < 0)
-      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)",
-                 i, j, _r, _c);
-    #endif
+#ifdef _DEBUG
+    if(i >= _r || i < 0 || j >= _c || j < 0)
+      Msg::Fatal("invalid index to access fullMatrix : %i %i (size = %i %i)", i,
+                 j, _r, _c);
+#endif
     return _data[i + _r * j];
   }
   void copy(const fullMatrix<scalar> &a, int i0, int ni, int j0, int nj,
@@ -570,11 +573,11 @@ class fullMatrix
   }
   void copy(const fullMatrix<scalar> &a)
   {
-    if (_data && !_own_data)
-      Msg::Fatal("fullMatrix::copy operation is prohibited for proxies, use setAll instead");
-    if (_r != a._r || _c != a._c) {
-      if(_data && _own_data)
-        delete [] _data;
+    if(_data && !_own_data)
+      Msg::Fatal("fullMatrix::copy operation is prohibited for proxies, use "
+                 "setAll instead");
+    if(_r != a._r || _c != a._c) {
+      if(_data && _own_data) delete[] _data;
       _r = a._r;
       _c = a._c;
       _data = new scalar[_r * _c];
@@ -593,24 +596,24 @@ class fullMatrix
   void mult(const fullMatrix<scalar> &b, fullMatrix<scalar> &c) const
 #if !defined(HAVE_BLAS)
   {
-    mult_naive(b,c);
+    mult_naive(b, c);
   }
 #endif
   ;
   void multTByT(const fullMatrix<scalar> &a)
   {
-    for (int i = 0; i < _r * _c; i++) _data[i] *= a._data[i];
+    for(int i = 0; i < _r * _c; i++) _data[i] *= a._data[i];
   }
-  void axpy(const fullMatrix<scalar> &x, scalar alpha=1.)
+  void axpy(const fullMatrix<scalar> &x, scalar alpha = 1.)
 #if !defined(HAVE_BLAS)
   {
     int n = _r * _c;
-    for (int i = 0; i < n; i++) _data[i] += alpha * x._data[i];
+    for(int i = 0; i < n; i++) _data[i] += alpha * x._data[i];
   }
 #endif
   ;
   void gemm_naive(const fullMatrix<scalar> &a, const fullMatrix<scalar> &b,
-                  scalar alpha=1., scalar beta=1.)
+                  scalar alpha = 1., scalar beta = 1.)
   {
     fullMatrix<scalar> temp(a.size1(), b.size2());
     a.mult_naive(b, temp);
@@ -619,10 +622,12 @@ class fullMatrix
     add(temp);
   }
   void gemm(const fullMatrix<scalar> &a, const fullMatrix<scalar> &b,
-            scalar alpha=1., scalar beta=1., bool transposeA = false, bool transposeB = false)
+            scalar alpha = 1., scalar beta = 1., bool transposeA = false,
+            bool transposeB = false)
 #if !defined(HAVE_BLAS)
   {
-    gemm_naive(transposeA ? a.transpose() : a, transposeB ? b.transpose() : b, alpha, beta);
+    gemm_naive(transposeA ? a.transpose() : a, transposeB ? b.transpose() : b,
+               alpha, beta);
   }
 #endif
   ;
@@ -633,8 +638,7 @@ class fullMatrix
   void setAll(const fullMatrix<scalar> &m)
 #if !defined(HAVE_BLAS)
   {
-    if (_r != m._r || _c != m._c )
-      Msg::Fatal("fullMatrix size does not match");
+    if(_r != m._r || _c != m._c) Msg::Fatal("fullMatrix size does not match");
     for(int i = 0; i < _r * _c; i++) _data[i] = m._data[i];
   }
 #endif
@@ -642,7 +646,8 @@ class fullMatrix
   void scale(const double s)
 #if !defined(HAVE_BLAS)
   {
-    if(s == 0.) // this is not really correct nan*0 (or inf*0) is expected to give nan
+    if(s == 0.) // this is not really correct nan*0 (or inf*0) is expected to
+                // give nan
       for(int i = 0; i < _r * _c; ++i) _data[i] = scalar(0.);
     else
       for(int i = 0; i < _r * _c; ++i) _data[i] *= s;
@@ -656,22 +661,19 @@ class fullMatrix
   inline void add(const fullMatrix<scalar> &m)
   {
     for(int i = 0; i < size1(); i++)
-      for(int j = 0; j < size2(); j++)
-        (*this)(i, j) += m(i, j);
+      for(int j = 0; j < size2(); j++) (*this)(i, j) += m(i, j);
   }
   inline void add(const fullMatrix<scalar> &m, const double &a)
   {
     for(int i = 0; i < size1(); i++)
-      for(int j = 0; j < size2(); j++)
-        (*this)(i, j) += a*m(i, j);
+      for(int j = 0; j < size2(); j++) (*this)(i, j) += a * m(i, j);
   }
   void mult(const fullVector<scalar> &x, fullVector<scalar> &y) const
 #if !defined(HAVE_BLAS)
   {
     y.scale(scalar(0.));
     for(int i = 0; i < _r; i++)
-      for(int j = 0; j < _c; j++)
-        y._data[i] += (*this)(i, j) * x(j);
+      for(int j = 0; j < _c; j++) y._data[i] += (*this)(i, j) * x(j);
   }
 #endif
   ;
@@ -679,8 +681,7 @@ class fullMatrix
 #if !defined(HAVE_BLAS)
   {
     for(int i = 0; i < _r; i++)
-      for(int j = 0; j < _c; j++)
-        y._data[i] += (*this)(i, j) * x(j);
+      for(int j = 0; j < _c; j++) y._data[i] += (*this)(i, j) * x(j);
   }
 #endif
   ;
@@ -688,14 +689,14 @@ class fullMatrix
   {
     fullMatrix<scalar> T(size2(), size1());
     for(int i = 0; i < size1(); i++)
-      for(int j = 0; j < size2(); j++)
-        T(j, i) = (*this)(i, j);
+      for(int j = 0; j < size2(); j++) T(j, i) = (*this)(i, j);
     return T;
   }
   inline void transposeInPlace()
   {
-    if(size1() != size2()){
-      Msg::Error("Not a square matrix (size1: %d, size2: %d)", size1(), size2());
+    if(size1() != size2()) {
+      Msg::Error("Not a square matrix (size1: %d, size2: %d)", size1(),
+                 size2());
     }
     scalar t;
     for(int i = 0; i < size1(); i++)
@@ -713,7 +714,7 @@ class fullMatrix
   }
 #endif
   ;
- bool luFactor(fullVector<int> &ipiv)
+  bool luFactor(fullVector<int> &ipiv)
 #if !defined(HAVE_LAPACK)
   {
     Msg::Error("LU factorization requires LAPACK");
@@ -721,7 +722,8 @@ class fullMatrix
   }
 #endif
   ;
- bool luSubstitute(const fullVector<scalar> &rhs, fullVector<int> &ipiv, fullVector<scalar> &result)
+  bool luSubstitute(const fullVector<scalar> &rhs, fullVector<int> &ipiv,
+                    fullVector<scalar> &result)
 #if !defined(HAVE_LAPACK)
   {
     Msg::Error("LU substitution requires LAPACK");
@@ -738,8 +740,8 @@ class fullMatrix
 #endif
   ;
   bool eig(fullVector<double> &eigenValReal, fullVector<double> &eigenValImag,
-           fullMatrix<scalar> &leftEigenVect, fullMatrix<scalar> &rightEigenVect,
-           bool sortRealPart=false)
+           fullMatrix<scalar> &leftEigenVect,
+           fullMatrix<scalar> &rightEigenVect, bool sortRealPart = false)
 #if !defined(HAVE_LAPACK)
   {
     Msg::Error("Eigenvalue computations requires LAPACK");
@@ -771,72 +773,51 @@ class fullMatrix
 #endif
   ;
 
-  void print(const std::string name = "", const std::string format = "%12.5E ") const
-  {
-    int ni = size1();
-    int nj = size2();
-    printf("double %s [ %d ][ %d ]= { \n", name.c_str(),ni,nj);
-    for(int I = 0; I < ni; I++){
-      printf("{  ");
-      for(int J = 0; J < nj; J++){
-        printf(format.c_str(), (*this)(I, J));
-	if (J!=nj-1)printf(",");
-      }
-      if (I!=ni-1)printf("},\n");
-      else printf("}\n");
-    }
-    printf("};\n");
-  }
+  void print(const std::string name = "", const std::string format = "") const;
 
-  void binarySave (FILE *f) const
+  void binarySave(FILE *f) const { fwrite(_data, sizeof(scalar), _r * _c, f); }
+  void binaryLoad(FILE *f)
   {
-    fwrite (_data, sizeof(scalar), _r*_c, f);
-  }
-  void binaryLoad (FILE *f)
-  {
-    if(fread (_data, sizeof(scalar), _r*_c, f) != (size_t)_r) return;
+    if(fread(_data, sizeof(scalar), _r * _c, f) != (size_t)_r) return;
   }
 
   // specific functions for dgshell
-  void mult_naiveBlock(const fullMatrix<scalar> &b, const int ncol, const int fcol,
-                       const int alpha, const int beta, fullVector<scalar> &c,
-                       const int row=0) const
+  void mult_naiveBlock(const fullMatrix<scalar> &b, const int ncol,
+                       const int fcol, const int alpha, const int beta,
+                       fullVector<scalar> &c, const int row = 0) const
   {
-    if(beta != 1)
-      c.scale(beta);
-    for(int j = fcol; j < fcol+ncol; j++)
-      for(int k = 0; k < _c ; k++)
-          c._data[j] += alpha*(*this)(row, k) * b(k, j);
+    if(beta != 1) c.scale(beta);
+    for(int j = fcol; j < fcol + ncol; j++)
+      for(int k = 0; k < _c; k++)
+        c._data[j] += alpha * (*this)(row, k) * b(k, j);
   }
   void multOnBlock(const fullMatrix<scalar> &b, const int ncol, const int fcol,
                    const int alpha, const int beta, fullVector<scalar> &c) const
 #if !defined(HAVE_BLAS)
   {
-    mult_naiveBlock(b,ncol,fcol,alpha,beta,c);
+    mult_naiveBlock(b, ncol, fcol, alpha, beta, c);
   }
 #endif
   ;
 
-  void multWithATranspose(const fullVector<scalar> &x, scalar alpha, scalar beta,
-                          fullVector<scalar> &y) const
+  void multWithATranspose(const fullVector<scalar> &x, scalar alpha,
+                          scalar beta, fullVector<scalar> &y) const
 #if !defined(HAVE_BLAS)
   {
     y.scale(beta);
     for(int j = 0; j < _c; j++)
-      for(int i = 0; i < _r; i++)
-        y._data[j] += alpha * (*this)(i, j) * x(i);
+      for(int i = 0; i < _r; i++) y._data[j] += alpha * (*this)(i, j) * x(i);
   }
 #endif
   ;
 
   void copyOneColumn(const fullVector<scalar> &x, const int ind) const
   {
-    int cind = _c*ind;
-    for(int i = 0; i < _r; i++)
-     _data[cind+i] = x(i);
+    int cind = _c * ind;
+    for(int i = 0; i < _r; i++) _data[cind + i] = x(i);
   }
 
-  bool getOwnData() {return _own_data;};
-  void setOwnData(bool ownData) {_own_data = ownData;};
+  bool getOwnData() const { return _own_data; };
+  void setOwnData(bool ownData) { _own_data = ownData; };
 };
 #endif
