@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _MATH_EVALUATOR_H_
 #define _MATH_EVALUATOR_H_
@@ -15,35 +15,36 @@
 
 #include "mathex.h"
 
-class mathEvaluator{
- private:
-  std::vector<smlib::mathex*> _expressions;
+class mathEvaluator {
+private:
+  std::vector<smlib::mathex *> _expressions;
   std::vector<double> _variables;
- public:
+
+public:
   // initialize one or more expressions depending on zero or more
   // variables. If an error occurs the vector of expressions is
   // cleared.
   mathEvaluator(std::vector<std::string> &expressions,
-                std::vector<std::string> &variables);
+                const std::vector<std::string> &variables);
   ~mathEvaluator();
   // evaluate the expression(s) using the given values and fill the
   // result vector. Returns true if the evaluation succeeded.
-  bool eval(std::vector<double> &values, std::vector<double> &res);
+  bool eval(const std::vector<double> &values, std::vector<double> &res);
 };
 
 #else
 
-class mathEvaluator{
- public:
+class mathEvaluator {
+public:
   mathEvaluator(std::vector<std::string> &expressions,
-                std::vector<std::string> &variables)
+                const std::vector<std::string> &variables)
   {
     Msg::Error("Gmsh must be compiled with MathEx support to evaluate math "
                "expressions");
     expressions.clear();
   }
-  ~mathEvaluator(){}
-  bool eval(std::vector<double> &values, std::vector<double> &res)
+  ~mathEvaluator() {}
+  bool eval(const std::vector<double> &values, std::vector<double> &res)
   {
     return false;
   }
