@@ -1,7 +1,7 @@
-// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #include <map>
 #include <stack>
@@ -12,18 +12,18 @@ class File_Position
 {
  public:
   int lineno;
-  gmshfpos_t position;
-  gmshFILE file;
+  fpos_t position;
+  FILE *file;
   std::string filename;
 };
 
-class mystack
+class FunctionManagerStack
 {
  public:
   std::stack<File_Position> s;
 };
 
-class mymap
+class FunctionManagerMap
 {
  public:
   std::map<std::string, File_Position> m;
@@ -33,8 +33,8 @@ FunctionManager *FunctionManager::instance = 0;
 
 FunctionManager::FunctionManager()
 {
-  functions = new mymap;
-  calls = new mystack;
+  functions = new FunctionManagerMap;
+  calls = new FunctionManagerStack;
 }
 
 FunctionManager *FunctionManager::Instance()
@@ -45,7 +45,12 @@ FunctionManager *FunctionManager::Instance()
   return instance;
 }
 
-int FunctionManager::enterFunction(const std::string &name, gmshFILE * f,
+void FunctionManager::clear()
+{
+  functions->m.clear();
+}
+
+int FunctionManager::enterFunction(const std::string &name, FILE **f,
                                    std::string &filename, int &lno) const
 {
   if(functions->m.find(name) == functions->m.end())
@@ -54,38 +59,40 @@ int FunctionManager::enterFunction(const std::string &name, gmshFILE * f,
   fpold.lineno = lno;
   fpold.filename = filename;
   fpold.file = *f;
-  gmshgetpos(fpold.file, &fpold.position);
+  fgetpos(fpold.file, &fpold.position);
   calls->s.push(fpold);
   File_Position fp = (functions->m)[name];
-  gmshsetpos(fp.file, &fp.position);
+  fsetpos(fp.file, &fp.position);
   *f = fp.file;
   filename = fp.filename;
   lno = fp.lineno;
   return 1;
 }
 
-int FunctionManager::leaveFunction(gmshFILE * f, std::string &filename, int &lno)
+int FunctionManager::leaveFunction(FILE **f, std::string &filename, int &lno)
 {
   if(!calls->s.size())
     return 0;
   File_Position fp;
   fp = calls->s.top();
   calls->s.pop();
-  gmshsetpos(fp.file, &fp.position);
+  fsetpos(fp.file, &fp.position);
   *f = fp.file;
   filename = fp.filename;
   lno = fp.lineno;
   return 1;
 }
 
-int FunctionManager::createFunction(const std::string &name, gmshFILE f,
+int FunctionManager::createFunction(const std::string &name, FILE *f,
                                     const std::string &filename, int lno)
 {
+  if(functions->m.find(name) != functions->m.end())
+    return 0;
   File_Position fp;
   fp.file = f;
   fp.filename = filename;
   fp.lineno = lno;
-  gmshgetpos(fp.file, &fp.position);
+  fgetpos(fp.file, &fp.position);
   (functions->m)[name] = fp;
   return 1;
 }
