@@ -1,25 +1,24 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _CUT_SPHERE_H_
 #define _CUT_SPHERE_H_
 
 #include "Levelset.h"
 
-extern "C"
-{
-  GMSH_Plugin *GMSH_RegisterCutSpherePlugin();
+extern "C" {
+GMSH_Plugin *GMSH_RegisterCutSpherePlugin();
 }
 
-class GMSH_CutSpherePlugin : public GMSH_LevelsetPlugin
-{
+class GMSH_CutSpherePlugin : public GMSH_LevelsetPlugin {
   double levelset(double x, double y, double z, double val) const;
   static double callback(int num, int action, double value, double *opt,
                          double step, double min, double max);
- public:
-  GMSH_CutSpherePlugin(){}
+
+public:
+  GMSH_CutSpherePlugin() {}
   std::string getName() const { return "CutSphere"; }
   std::string getShortHelp() const
   {
@@ -27,7 +26,7 @@ class GMSH_CutSpherePlugin : public GMSH_LevelsetPlugin
   }
   std::string getHelp() const;
   int getNbOptions() const;
-  StringXNumber *getOption(int iopt);  
+  StringXNumber *getOption(int iopt);
   PView *execute(PView *);
 
   static double callbackX(int, int, double);
