@@ -1,22 +1,20 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 
 #ifndef _FIELD_FROM_AMPLITUDE_PHASE_H_
 #define _FIELD_FROM_AMPLITUDE_PHASE_H_
 
 #include "Plugin.h"
 
-extern "C"
-{
-  GMSH_Plugin *GMSH_RegisterFieldFromAmplitudePhasePlugin();
+extern "C" {
+GMSH_Plugin *GMSH_RegisterFieldFromAmplitudePhasePlugin();
 }
 
-class GMSH_FieldFromAmplitudePhasePlugin : public GMSH_PostPlugin
-{
- public:
-  GMSH_FieldFromAmplitudePhasePlugin(){}
+class GMSH_FieldFromAmplitudePhasePlugin : public GMSH_PostPlugin {
+public:
+  GMSH_FieldFromAmplitudePhasePlugin() {}
   std::string getName() const { return "FieldFromAmplitudePhase"; }
   std::string getShortHelp() const
   {
@@ -26,7 +24,7 @@ class GMSH_FieldFromAmplitudePhasePlugin : public GMSH_PostPlugin
   int getNbOptions() const;
   StringXNumber *getOption(int iopt);
   int getNbOptionsStr() const;
-  StringXString* getOptionStr(int iopt);  
+  StringXString *getOptionStr(int iopt);
   PView *execute(PView *);
 };
 
