@@ -1,9 +1,9 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2018 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
-// bugs and problems to the public mailing list <gmsh@geuz.org>.
+// bugs and problems to the public mailing list <gmsh@onelab.info>.
 //
-// Contributed by Matti Pellikka <matti.pellikka@tut.fi>.
+// Contributed by Matti Pellikka <matti.pellikka@microsoft.com>.
 
 #ifndef _HOMOLOGY_POST_PROCESSING_H_
 #define _HOMOLOGY_POST_PROCESSING_H_
@@ -14,15 +14,13 @@
 
 #if defined(HAVE_KBIPACK)
 
-extern "C"
-{
-  GMSH_Plugin *GMSH_RegisterHomologyPostProcessingPlugin();
+extern "C" {
+GMSH_Plugin *GMSH_RegisterHomologyPostProcessingPlugin();
 }
 
-class GMSH_HomologyPostProcessingPlugin : public GMSH_PostPlugin
-{
- public:
-  GMSH_HomologyPostProcessingPlugin(){}
+class GMSH_HomologyPostProcessingPlugin : public GMSH_PostPlugin {
+public:
+  GMSH_HomologyPostProcessingPlugin() {}
   std::string getName() const { return "HomologyPostProcessing"; }
   std::string getShortHelp() const
   {
@@ -35,9 +33,9 @@ class GMSH_HomologyPostProcessingPlugin : public GMSH_PostPlugin
   int getNbOptionsStr() const;
   StringXString *getOptionStr(int iopt);
   PView *execute(PView *);
-  bool parseStringOpt(int stringOpt, std::vector<int>& intList);
-  bool invertIntegerMatrix(std::vector<int>& matrix);
-  int detIntegerMatrix(std::vector<int>& matrix);
+  bool parseStringOpt(int stringOpt, std::vector<int> &intList);
+  bool invertIntegerMatrix(std::vector<int> &matrix);
+  int detIntegerMatrix(std::vector<int> &matrix);
 };
 
 #endif
